@@ -2,7 +2,7 @@
                           fitdlg.cpp  -  description
                              -------------------
     begin                : Tue May 25 1999
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -15,40 +15,44 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <stdlib.h>
 #include <math.h>
-#include <qgroupbox.h>
+#include <stdlib.h>
+#include <qbuttongroup.h>
+#include <qcheckbox.h>
+#include <qdir.h>
 #include <qfileinfo.h>
+#include <qgroupbox.h>
+#include <qlabel.h>
+#include <qlayout.h>
 #include <qlistbox.h>
 #include <qlineedit.h>
-#include <qcheckbox.h>
-#include <qlabel.h>
 #include <qpainter.h>
-#include <qdir.h>
-#include <qlayout.h>
+#include <qradiobutton.h>
+#include <qregexp.h>
 #include <qtable.h>
 #include <qtimer.h>
-#include <qregexp.h>
 #include <kapp.h>
-#include <klocale.h>
-#include <kurl.h>
 #include <kcolorbtn.h>
-#include <kseparator.h>
 #include <kconfig.h>
-#include <sourcewidget.h>
-#include <kmessagebox.h>
 #include <kglobal.h>
-#include "fitdlg.h"
-#include "lmfit.h"
-#include "kpldoc.h"
-#include "frameitem.h"
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <kseparator.h>
+#include <kurl.h>
 #include "arrayitem.h"
+#include "choosefuncdlg.h"
+#include "editparameterdlg.h"
+#include "fitdlg.h"
+#include "frameitem.h"
 #include "funitem.h"
-#include "utils.h"
+#include "kpldoc.h"
 #include "kpldoubleedit.h"
-#include "kplspinbox.h"
-#include "residualsdlg.h"
 #include "kplgraph.h"
+#include "kplspinbox.h"
+#include "lmfit.h"
+#include "residualsdlg.h"
+#include "sourcewidget.h"
+#include "utils.h"
 
 CorrDlg::CorrDlg(QWidget* _parent, int np, const int* ip, const double* corr) :
   KDialog(_parent, 0, true)
@@ -86,7 +90,7 @@ ErrModDlg::ErrModDlg(QWidget* _parent, KplDoc* model,
 {
   errt = new Kpl::DataErrorStruct(*err);
   QFrame* frame = plainPage();
-  QGridLayout* grid = new QGridLayout(frame, 3, 2, 0, spacingHint());
+  QGridLayout* grid = new QGridLayout(frame, 4, 2, 0, spacingHint());
   grid->addWidget(new QLabel(i18n("Library"), frame), 0, 0);
   QHBoxLayout* hbox = new QHBoxLayout();
   grid->addLayout(hbox, 0, 1);
@@ -100,10 +104,20 @@ ErrModDlg::ErrModDlg(QWidget* _parent, KplDoc* model,
   hbox->addWidget(func = new QLineEdit(err->errModName, frame));
   hbox->addWidget(b = new QPushButton("...", frame));
   b->setFixedWidth(28);
+  grid->addWidget(new QLabel(i18n("Argument"), frame), 2, 0);
+  grid->addLayout(hbox = new QHBoxLayout(), 2, 1);
+  bg = new QButtonGroup();
+  QRadioButton* r;
+  hbox->addWidget(r = new QRadioButton(i18n("x column"), frame));
+  bg->insert(r);
+  hbox->addWidget(r = new QRadioButton(i18n("y column"), frame));
+  bg->insert(r);
+  ((QRadioButton*) bg->find(err->errModArg))->setChecked(true);
+  hbox->addItem(new QSpacerItem(10, 10, QSizePolicy::Expanding));
   setHelp("ERRORMODEL");
   connect(b, SIGNAL(clicked()), SLOT(slotFunc()));
-  grid->addWidget(new QLabel(i18n("Parameter"), frame), 2, 0);
-  grid->addLayout(hbox = new QHBoxLayout(), 2, 1);
+  grid->addWidget(new QLabel(i18n("Parameter"), frame), 3, 0);
+  grid->addLayout(hbox = new QHBoxLayout(), 3, 1);
   hbox->addWidget(b = new QPushButton(i18n("Edit"), frame));
   connect(b, SIGNAL(clicked()), SLOT(slotEditParameter()));
   hbox->addWidget(b = new QPushButton(i18n("Load"), frame));
@@ -119,7 +133,7 @@ ErrModDlg::~ErrModDlg()
 
 void ErrModDlg::slotFile()
 {
-  Utils::getFile(fileName, m);
+  m->getFile(fileName);
 }
 
 void ErrModDlg::slotFunc()
@@ -137,7 +151,7 @@ void ErrModDlg::slotFunc()
 
 void ErrModDlg::slotGetParameter()
 {
-  Utils::getPar(this, errt->pErrMod, m);
+  m->getPar(this, errt->pErrMod);
 }
 
 void ErrModDlg::slotEditParameter()
@@ -148,13 +162,14 @@ void ErrModDlg::slotEditParameter()
 
 void ErrModDlg::slotSaveParameter()
 {
-  Utils::saveFunPar(this, errt->pErrMod, m);
+  m->saveFunPar(this, errt->pErrMod);
 }
 
 void ErrModDlg::slotOk()
 {
   errt->errModPath = fileName->text();
   errt->errModName = func->text();
+  errt->errModArg = bg->id(bg->selected());
   lt_dlhandle hErrMod = 0;
   double (*fkt)(double, const double *);
   if (FunItem::getFuncAddr(errt->errModPath, errt->errModName, &hErrMod, &fkt))
@@ -196,6 +211,8 @@ FitDlg::FitDlg(QWidget* _parent, KplDoc* model, QList<ArrayItem>* ad0,
                                                  aut->err.fitErrCol);
       else
         err[i].fitErrCol = false;
+      err[i].errModArg = config->readNumEntry("ErrorModelArgument",
+                                              aut->err.errModArg);
     }
   }
   FunItem* ff = fd->first();
@@ -335,6 +352,7 @@ void FitDlg::getValues(bool ok)
       for (i = 0; i < KPL_NPMAX; i++)
         s1.insert(i, m->number(err[j].pErrMod[i]));
       config->writeEntry("ErrorModelParam", s1, ' ');
+      config->writeEntry("ErrorModelArgument", err[j].errModArg);
     }
   }
   for (j = 0; j < (int) fd->count(); j++)
@@ -355,7 +373,7 @@ void FitDlg::slotMessage(const QString& msg)
 
 void FitDlg::slotGetParameter()
 {
-  Utils::getPar(this, m->pLast, m);
+  m->getPar(this, m->pLast);
   for (int i = 0; i < KPL_NPMAX; i++) {
     par[i]->setValue(m->pLast[i]);
     m->pErr[i] = 0.0;
@@ -366,8 +384,8 @@ void FitDlg::slotGetParameter()
 void FitDlg::slotSaveParameter()
 {
   KURL url;
-  if (Utils::getWriteURL(this, url, "*.par\n*", m))
-    Utils::saveParameter(this, url, m->pLast, m->pErr, m);
+  if (m->getWriteURL(this, url, "*.par\n*"))
+    m->saveParameter(this, url, m->pLast, m->pErr);
 }
 
 void FitDlg::errColToggled(bool state)
@@ -507,7 +525,8 @@ void FitDlg::fit()
                                &fktErrMod))
         return;
       for (i = 0; i < a->n; i++)
-        lm->sig[j++] = fktErrMod(a->x[a->iy][a->istart + i], err[k].pErrMod);
+        lm->sig[j++] = fktErrMod(a->x[err[k].errModArg ? a->iy : a->ix]
+                                     [a->istart + i], err[k].pErrMod);
       lt_dlclose(hErrMod);
     }
   }
@@ -651,7 +670,7 @@ void FitDlg::fit()
       KURL url = m->URL();
       QFileInfo fi(url.path());
       url.setPath(fi.dirPath(true) + "/" + fi.baseName() + ".par");
-      Utils::saveParameter(this, url, m->pLast, m->pErr, m);
+      m->saveParameter(this, url, m->pLast, m->pErr);
     }
   }
   if (dlgMode & ShowDlg) {
@@ -684,7 +703,7 @@ void FitDlg::slotUser2()
   items.append(fd);
   int j = 0;
   for (ArrayItem* a = ad->first(); a; a = ad->next()) {
-    ArrayItem* ar = new ArrayItem(true, a->symb, "0", 1.0, 1.0, 0, 1, 2,
+    ArrayItem* ar = new ArrayItem(true, false, a->symb, "0", 1.0, 1.0, 0, 1, 2,
                                   0, a->n, 1, "", true);
     ar->color = a->color;
     for (int i = 0; i < a->n; i++) {
