@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Sun Apr 25 1999
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -24,14 +24,15 @@
 
 #include <kdialogbase.h>
 
-class KplDoc;
-class KColorButton;
 class QLineEdit;
 class QListBox;
+class KColorButton;
+class BrushButton;
 class FunItem;
-class ParFunItem;
+class KplDoc;
 class KplDoubleEdit;
 class KplSpinBox;
+class ParFunItem;
 
 /**
   * Function settings dialog class. Provides a dialog for the settings
@@ -76,6 +77,7 @@ protected:
   KplDoc* m;
   FunItem *fd, *fdt;
   QLineEdit *fileName, *func;
+  BrushButton* fill;
   KplDoubleEdit *exMin, *exMax, *exStep, *efx, *efy;
   KplSpinBox *symb, *eRelSize;
   KColorButton* colData;
@@ -134,6 +136,7 @@ protected:
   KplDoc* m;
   ParFunItem *fd, *fdt;
   QLineEdit *fileNamex, *funcx, *fileNamey, *funcy;
+  BrushButton* fill;
   KplDoubleEdit *etMin, *etMax, *etStep, *efx, *efy;
   KplSpinBox *symb, *eRelSize;
   KColorButton* colData;
