@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Sun Apr 25 1999
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -86,6 +86,12 @@ public:
   ~ItemDlg();
   /** Updates the item list. */
   void updItemList();
+  /**
+   * Displays a popup menu.
+   * @param it index of clicked item.
+   * @param p point where popup menu will appear.
+   */
+  void showPopup(int it, const QPoint& p);
 
 protected:
   /**
@@ -138,7 +144,7 @@ protected slots:
   /**
    * Displays a popup menu.
    * @param item pointer to clicked QCheckListItem.
-   * @param point where popup menu will appear.
+   * @param p point where popup menu will appear.
    */
   void slotPopup(QListViewItem* item, const QPoint& p);
   /**
@@ -146,16 +152,31 @@ protected slots:
    * @param item pointer to clicked QCheckListItem.
    */
   void slotExecuted(QListViewItem* item);
-  /** Updates item structure and repaints view. */
-  void slotStateChanged(KplCheckListItem* item, bool bState);
+  /**
+   * Updates item structure and repaints view after change of active state.
+   * @param item pointer to clicked KplCheckListItem.
+   * @param active true for visible item.
+   */
+  void slotStateChanged(KplCheckListItem* item, bool active);
+  /**
+   * Updates item structure and repaints view after change of open state.
+   * @param item pointer to clicked KplCheckListItem.
+   * @param open true for open in list view.
+   */
+  void slotOpenChanged(KplCheckListItem* item, bool open);
   /** Toggles active state of item. */
   void slotToggleState();
-  /** Enables/disables move buttons. */
+  /**
+   * Enables/disables move buttons.
+   * @param item pointer to clicked QCheckListItem.
+   */
   void slotSelectionChanged(QListViewItem* item);
   /** Moves item up in sequence. */
   void slotMoveUp();
   /** Moves item down in sequence. */
   void slotMoveDown();
+  /** Sets widget geometry. */
+  void setPosition();
 
 signals:
   /**
