@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Sun Apr 25 1999
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -20,14 +20,27 @@
  ***************************************************************************/
 
 #include <math.h>
+#include <qimage.h>
 #include <qpaintdevicemetrics.h>
 #include <qpixmap.h>
 #include <kapp.h>
+#include <kio/netaccess.h>
+#include <kmessagebox.h>
+#include <kurl.h>
 #include "kgraph.h"
+#include "utils.h"
+
+#if QT_VERSION >= 0x030200
+#include <qfontdatabase.h>
+#endif
 
 //#define NO_CLIPPING
 
-const char* KGraph::face[] = {"Helvetica", "Symbol", "itc zapf dingbats"};
+const char* KGraph::face[] = {"Helvetica", "Symbol", "Dingbats"};
+
+const int KGraph::pattern[] = {NoBrush, SolidPattern, HorPattern,
+                               VerPattern, CrossPattern, BDiagPattern,
+                               FDiagPattern, DiagCrossPattern};
 
 KGraph::KGraph() : minx(0), maxx(0), miny(0), maxy(0), kxcur(0), kycur(0),
  ndigx(0), ndigy(0), minx0(0), miny0(0), pFactor(0), textdir(0.0),
@@ -150,11 +163,15 @@ void KGraph::plInit(const QPaintDevice* pd, int offLeft, int offTop)
   eraseRect(window());
   if (pd->devType() == QInternal::Printer) {
     devType = Printer;
+#if KDE_VERSION_MAJOR > 2
+    pFactor = 1;
+#else
     pFactor = 4;
     QRect r = window();
     r.setSize(pFactor * r.size());
     setWindow(r);
-  } else 
+#endif
+  } else
     pFactor = 1;
   fDpi = 75.0 / pdm.logicalDpiY();
   scalx = pFactor * pdm.logicalDpiX() / 2.54;
@@ -177,7 +194,7 @@ void KGraph::plInit(const QPaintDevice* pd, int offLeft, int offTop)
   relang = 0.0;
   relsin = 0.0;
   relcos = 1.0;
-  relSize = 1.0;
+  relSiz = 1.0;
   textdir = 0.0;
   ndigx = ndigy = -1;
   linetype(1);
@@ -185,10 +202,7 @@ void KGraph::plInit(const QPaintDevice* pd, int offLeft, int offTop)
   pen2.setWidth((maxx - minx) / 480);
   pen3.setWidth((maxx - minx) / 339);
   setPen(pen3);
-  QFont fnt = font();
-  fnt.setFamily(face[0]);
-  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * maxx), 1));
-  setFont(fnt);
+  setFont(QFont(face[0], QMAX(qRound(0.0355 * fDpi * maxx), 1)));
   move(xcur, ycur);
 #ifndef NO_CLIPPING
   setClipRect(-offLeft, offTop, pdm.width(), pdm.height());
@@ -241,14 +255,14 @@ void KGraph::setCol(unsigned icol1, unsigned icol2, unsigned icol3)
 
 double KGraph::setRelSize(double r)
 {
-  double old = relSize;
-  relSize = r;
-  pen1.setWidth(qRound(0.00416667 * relSize * (maxx - minx)));
-  pen2.setWidth(qRound(0.00208333 * relSize * (maxx - minx)));
-  pen3.setWidth(qRound(0.00295 * relSize * (maxx - minx)));
+  double old = relSiz;
+  relSiz = r;
+  pen1.setWidth(qRound(0.00416667 * relSiz * (maxx - minx)));
+  pen2.setWidth(qRound(0.00208333 * relSiz * (maxx - minx)));
+  pen3.setWidth(qRound(0.00295 * relSiz * (maxx - minx)));
   setPen(pen3);
   QFont fnt = font();
-  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * relSize * (maxx - minx)), 1));
+  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * relSiz * (maxx - minx)), 1));
   setFont(fnt);
   return old;
 }
@@ -267,13 +281,13 @@ void KGraph::Window(double xmi, double w, double ymi, double h)
   ycur = ymin;
   move(xcur, ycur);
   dcur = 0.0;
-  int _dx = qRound(0.00416667 * relSize * (maxx - minx));
+  int _dx = qRound(0.00416667 * relSiz * (maxx - minx));
   pen1.setWidth(_dx);
-  pen2.setWidth(qRound(0.00208333 * relSize * (maxx - minx)));
-  pen3.setWidth(qRound(0.00295 * relSize * (maxx - minx)));
+  pen2.setWidth(qRound(0.00208333 * relSiz * (maxx - minx)));
+  pen3.setWidth(qRound(0.00295 * relSiz * (maxx - minx)));
   setPen(pen3);
   QFont fnt = font();
-  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * relSize * (maxx - minx)), 1));
+  fnt.setPointSize(QMAX(qRound(0.0355 * fDpi * relSiz * (maxx - minx)), 1));
   setFont(fnt);
 #ifndef NO_CLIPPING
   setClipRect((minx + (_dx >> 1)) / pFactor , (maxy + (_dx >> 1)) / pFactor,
@@ -455,9 +469,9 @@ void KGraph::raster(double xtic, double ytic, int intx, int inty, int mode,
     QString label;
     double yB = 0.0, yT = 0.0;
     if (xBottom)
-      yB = ymin + 0.03225 * relSize * (xmax - xmin) * xscal / yscal;
+      yB = ymin + 0.03225 * relSiz * (xmax - xmin) * xscal / yscal;
     if (xTop)
-      yT = ymax - 0.03225 * relSize * (xmax - xmin) * xscal / yscal;
+      yT = ymax - 0.03225 * relSiz * (xmax - xmin) * xscal / yscal;
     int ix, iy;
     bool e = (logx && (xtic > 9.9) && ((xmax > 2.999999) || (xmin < -2.999999)));
     int nd = ndigx;
@@ -498,10 +512,10 @@ void KGraph::raster(double xtic, double ytic, int intx, int inty, int mode,
     }
     double xL = 0.0, xR = 0.0;
     if (yLeft)
-      xL = xmin - 0.025 * relSize * (xmax - xmin);
+      xL = xmin - 0.025 * relSiz * (xmax - xmin);
     if (yRight)
-      xR = xmax + 0.025 * relSize * (xmax - xmin);
-    dy = -0.0125 * relSize * (xmax - xmin) * xscal / yscal;
+      xR = xmax + 0.025 * relSiz * (xmax - xmin);
+    dy = -0.0125 * relSiz * (xmax - xmin) * xscal / yscal;
     e = (logy && (ytic > 9.9) && ((ymax > 2.999999) || (ymin < -2.999999)));
     nd = ndigy;
     double ye = ymax + 1.0e-3 * (ymax - ymin);
@@ -582,17 +596,17 @@ void KGraph::text(const QString& str, int ix, int iy, int iorg, QRect* bRect)
       }
       drawText(0, 0, 0, 0, tf, str, -1, bRect);
     } else {
-      const char fontcode[] = "NHBOSZI", levcode[] = "NUD";
+      const QString fontcode = "NHBOSZI";
+      const QString levcode = "NUD";
       const int iface[7] = {0, 0, 0, 0, 1, 2, 0};
       const int weight[7] = {QFont::Normal, QFont::Normal, QFont::Bold,
                              QFont::Normal, QFont::Normal, QFont::Normal,
                              QFont::Bold};
       const bool italic[7] = {false, false, false, true, false, false, true};
       const double ys[2] = {0.6, -0.4};
-      const char *c;
       int ix2 = 0;
       tf = AlignLeft | AlignVCenter | DontClip;
-      double dr = 0.025 * relSize * (xmax - xmin) * xscal;
+      double dr = 0.025 * relSiz * (xmax - xmin) * xscal;
       int j0 = (iorg == 1) ? 1 : 0;
       int idx = 0;
       int idy;
@@ -603,10 +617,15 @@ void KGraph::text(const QString& str, int ix, int iy, int iorg, QRect* bRect)
           if (iorg != 1)
             ix2 = -idx;
         }
-        const char* s = str.latin1();
+        QString s = str;
         int ilev = 0;
-        while (*s) {
-          if ((l = ((c = strchr(s, '#'))) ? (int) (c - s) : strlen(s))) {
+        while (s.length()) {
+          QString c;
+          if ((l = s.find('#')) < 0)
+            l = s.length();
+          else
+            c = s.mid(l);
+          if (l) {
             QRect r;
             if (j == 0) {
               r = boundingRect(0, 0, maxx, maxy,
@@ -620,37 +639,45 @@ void KGraph::text(const QString& str, int ix, int iy, int iorg, QRect* bRect)
                 *bRect = bRect->unite(r);
             }
           }
-          if (c) {
-            if (strlen(c) < 3)
+          if (!c.isNull()) {
+            if (c.length() < 3)
               break;
-            s = c + 3;
-            char code;
-            if ((code = *(++c)) > 96)
-              code -= (char) 32;
-            const char* f;
-            if (!(f = strchr(fontcode, code)))
-              break;
-            int ifnt = (int) (f - fontcode);
-            if ((code = *(++c)) > 96)
-              code -= (char) 32;
-            if (!(f = strchr(levcode, code)))
-              break;
-            setFont(QFont(face[iface[ifnt]],
-                          QMAX(qRound(((ilev = (int) (f - levcode)) ?
-                                       0.0251 : 0.0355) *
-                                       relSize * fDpi * (maxx - minx)), 1),
-#if KDE_VERSION_MAJOR > 2
-                          weight[ifnt], italic[ifnt]));
-#else
-                          weight[ifnt], italic[ifnt],
-                          iface[ifnt] ? QFont::AnyCharSet : QFont::Latin1));
+            s = c.mid(3);
+            int ifnt = fontcode.find(c.at(1).upper());
+            ilev = levcode.find(c.at(2).upper());
+            if ((ifnt >= 0) && (ilev >= 0)) {
+              int k = iface[ifnt];
+#if QT_VERSION >= 0x030200
+              if (k == 2)
+                QFontDatabase().styles(face[k]);
 #endif
+              QFont fnt(face[k],
+                        QMAX(qRound((ilev ? 0.0251 : 0.0355) * relSiz * fDpi *
+                                    (maxx - minx)), 1),
+                        weight[ifnt], italic[ifnt]);
+#if KDE_VERSION_MAJOR < 3
+              fnt.setCharSet(k ? QFont::AnyCharSet : QFont::Latin1);
+#endif
+              if (k > 0) {
+#if KDE_VERSION_MAJOR < 3
+                fnt.setStyleHint(QFont::AnyStyle, QFont::PreferBitmap);
+#else
+                fnt.setStyleStrategy(QFont::PreferBitmap);
+#endif
+                if (QFontInfo(fnt).family().find(QString(face[k]), 0, false))
+#if KDE_VERSION_MAJOR < 3
+                  fnt.setStyleHint(QFont::AnyStyle, QFont::PreferDefault);
+#else
+                  fnt.setStyleStrategy(QFont::PreferDefault);
+#endif
+              }
+              setFont(fnt);
+            }
           } else
-            s += l;
+            s.remove(0, l);
         }
-        setFont(QFont(face[iface[0]],
-                QMAX(qRound(0.0355 * relSize * fDpi * (maxx - minx)), 1),
-                weight[0], italic[0]));
+        setFont(QFont(face[0],
+                QMAX(qRound(0.0355 * relSiz * fDpi * (maxx - minx)), 1)));
       }
     }
     setPen(pen3);
@@ -664,7 +691,7 @@ void KGraph::textcm(const QString& str, double x, double y, int al,
                     QRect* bRect)
 {
   double xa, ya;
-  double dh = 0.013 * relSize * (maxx - minx) / (scalx * factor);
+  double dh = 0.013 * relSiz * (maxx - minx) / (scalx * factor);
   cm2r(x - dh * relsin, y + dh * relcos, 1.0, 1.0, &xa, &ya);
   int ix, iy;
   r2i(logx ? log10(xa) : xa, logy ? log10(ya) : ya, &ix, &iy);
@@ -677,9 +704,10 @@ void KGraph::arrow(double x, double y, double dir, double len,
   QWMatrix wm = worldMatrix();
   QPoint p = fromcm(x, y);
   translate(double(p.x()), double(p.y()));
+  QPainter::scale(1.0, -scaly / scalx);
   if (dir)
     rotate(-dir);
-  double dx = 0.03333 * relSize * (maxx - minx);
+  double dx = 0.03333 * relSiz * (maxx - minx);
   int idx = -qRound(dx);
   int idy = QMAX(1, qRound(0.2 * dx));
   QPointArray a;
@@ -716,7 +744,7 @@ void KGraph::arc(double x, double y, double w, double h, double a,
 }
 
 void KGraph::rectangle(double x, double y, double w, double h,
-                       bool filled, double dir, QRect* bRect)
+                       int fillStyle, double dir, QRect* bRect)
 {
   QWMatrix wm = worldMatrix();
   QPoint p = fromcm(x, y);
@@ -725,12 +753,12 @@ void KGraph::rectangle(double x, double y, double w, double h,
     rotate(-dir);
   int idy = qRound(factor * scaly * h);
   QRect r(0, idy, qRound(factor * scalx * w), -idy);
-  if (filled) {
-    setBrush(pen3.color());
+  if (fillStyle) {
+    setBrush(QBrush(pen3.color(), (BrushStyle) pattern[fillStyle]));
     setPen(NoPen);
   }
   drawRect(r);
-  if (filled) {
+  if (fillStyle) {
     setBrush(NoBrush);
     setPen(pen3);
   }
@@ -740,7 +768,7 @@ void KGraph::rectangle(double x, double y, double w, double h,
 }
 
 void KGraph::ellipse(double x, double y, double w, double h,
-                     bool filled, double dir, QRect* bRect)
+                     int fillStyle, double dir, QRect* bRect)
 {
   QWMatrix wm = worldMatrix();
   QPoint p = fromcm(x, y);
@@ -750,12 +778,12 @@ void KGraph::ellipse(double x, double y, double w, double h,
   int iw = qRound(factor * scalx * w);
   int ih = -qRound(factor * scaly * h);
   QRect r(-iw / 2, -ih / 2, iw, ih);
-  if (filled) {
-    setBrush(pen3.color());
+  if (fillStyle) {
+    setBrush(QBrush(pen3.color(), (BrushStyle) pattern[fillStyle]));
     setPen(NoPen);
   }
   drawEllipse(r);
-  if (filled) {
+  if (fillStyle) {
     setBrush(NoBrush);
     setPen(pen3);
   }
@@ -764,11 +792,72 @@ void KGraph::ellipse(double x, double y, double w, double h,
   setWorldMatrix(wm);
 }
 
+void KGraph::image(const KURL& url, double x, double y, double sx, double sy,
+                   QRect* bRect)
+{
+  QString fn;
+  double x1, x2, y1, y2;
+  if (Utils::boundingBox(url, fn, &x1, &y1, &x2, &y2)) {
+    double w = 0.0352778 * sx * (x2 - x1);
+    double h = 0.0352778 * sy * (y2 - y1);
+    int iw = qRound(factor * scalx * w);
+    int ih = qRound(-factor * scaly * h);
+    QPoint p = fromcm(x, y + h);
+#if KDE_VERSION_MAJOR < 3
+    QImage image(fn);
+    drawImage(p, image.smoothScale(iw, ih));
+#else
+    QImageIO iio;
+    iio.setFileName(fn);
+    QString s = QString::number(iw) + ":" + QString::number(ih);
+    iio.setParameters(s.latin1());
+    iio.read();
+    drawImage(p, iio.image());
+#endif
+    if (bRect)
+      *bRect = QRect(p, QSize(iw, ih));
+    if (!url.isLocalFile())
+      KIO::NetAccess::removeTempFile(fn);
+  } else
+    KMessageBox::error(0, url.url());
+}
+
+void KGraph::scaleBar(const QString& str, double x, double y, int orientation,
+                      double len, QRect* bRect)
+{
+  QWMatrix wm = worldMatrix();
+  QPoint p = fromcm(x, y);
+  translate(double(p.x()), double(p.y()));
+  QPainter::scale(1.0, -scaly / scalx);
+  if (orientation)
+    rotate(-90.0);
+  int idy = qRound(0.005 * (maxx - minx));
+  int idx = qRound(factor * scalx * len);
+  setPen(pen1);
+  drawLine(0, 0, idx, 0);
+  drawLine(0, -idy, 0, idy);
+  drawLine(idx, -idy, idx, idy);
+  QRect r2;
+  if (bRect)
+    r2 = xForm(QRect(0, -idy, idx, idy + idy));
+  setPen(pen3);
+  setWorldMatrix(wm);
+  if (orientation)
+    textcm(str, x  + 2 * idy / (factor * scalx),
+           y + 0.5 * len + 0.014 * relSiz * (maxx - minx) / (factor * scaly),
+           1, bRect);
+  else
+    textcm(str, x + 0.5 * len, y - (2 + relSiz) * idy / (factor * scaly),
+           2, bRect);
+  if (bRect)
+    *bRect = bRect->unite(r2);
+}
+
 void KGraph::letH(const QString& str, bool xTop)
 {
   int ix, iy;
   r2i(0.5 * (xmin + xmax),
-      ymax - (xTop ? 0.1875 : 0.04275) * relSize * (xmax - xmin) * xscal / yscal,
+      ymax - (xTop ? 0.1875 : 0.04275) * relSiz * (xmax - xmin) * xscal / yscal,
       &ix, &iy);
   text(str, ix, iy, 2);
 }
@@ -832,7 +921,7 @@ double KGraph::incLog(double* x, double f)
 }
 
 void KGraph::plArray(const double *x, const double *y, double fx, double fy,
-                     int n, bool clip)
+                     int n, int fillStyle, bool clip)
 {
   if (kSymb == -17)
     return;
@@ -840,80 +929,94 @@ void KGraph::plArray(const double *x, const double *y, double fx, double fy,
   setClipping(clip);
 #endif
   double xa, ya;
-  if (kSymb > 0) {
-    linetype(kSymb);
-    xa = fx * x[0];
-    ya = fy * y[0];
-    bool b = false;
-    if (((!logx) || (xa > 0)) && ((!logy) || (ya > 0))) {
-      move(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
-      b = true;
-    }
-    for (int i = 1; i < n; i++) {
-      xa = fx * x[i];
-      ya = fy * y[i];
-      if (((!logx) || (xa > 0)) && ((!logy) || (ya > 0)))
-        if (b)
-          draw(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
-        else {
-          move(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
-          b = true;
-        }
-    }
-    linetype(1);
-  } else {
-    int k = (-kSymb) % 17;
-    QPointArray ap(10);
-    if ((k > 10) || (k == 1)) {
-      setBrush(pen3.color());
-      setPen(NoPen);
-    }
-    const int ax[48] = {0, -2, 0, 2, 0, 2, -2, -2, 2, 2, 0, -2, 2, 0,
-                        2, -2, 2, -2, 2, 2, 1, -1, -2, -2, -1, 1, 2, 2,
-                        2, -2, 0, -2, 2, 0, 0, 0, -2, 2, 2, 0, -2, 0, 0,
-                        0, -2, 2, 0, 0},
-              ay[48] = {2, 0, -2, 0, 2, 2, 2, -2, -2, 2, 2, -1, -1, 2,
-                        2, 2, -2, -2, 2, 1, 2, 2, 1, -1, -2, -2, -1, 1,
-                        2, -2, 0, 2, -2, 2, -2, 0, 0, 0, 2, 0, 2, 0, -2,
-                        2, 0, 0, 2, -2},
-              as[10] = {0, 5, 10, 14, 19, 28, 33, 38, 43, 48};
-    int j, l = 0, r;
-    double f = 0.005 * relSize * (maxx - minx);
+  if (fillStyle) {
+    setBrush(QBrush(pen3.color(), (BrushStyle) pattern[fillStyle]));
+    setPen(NoPen);
+    QPointArray ap(n);
+    int kx, ky;
+    int l = 0;
     for (int i = 0; i < n; i++) {
       xa = fx * x[i];
       ya = fy * y[i];
-      if (((!logx) || (xa > 0)) && ((!logy) || (ya > 0))) {
-        move(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
-        switch (k) {
-          case 0:
-          case 11:
-            r = qRound(f + f);
-            drawEllipse(kxcur - r, kycur - r, r + r, r + r);
-            break;
-          case 1:
-            r = qRound(0.4 * f);
-            drawEllipse(kxcur - r, kycur - r, r + r, r + r);
-            break;
-          default:
-            r = (k < 12) ? k : k - 10;
-            for (j = as[r - 2]; j < as[r - 1]; j++) {
-              l = j - as[r - 2];
-              if (k < 12)
-                if (l)
-                  lineTo(qRound(kxcur + f * ax[j]), qRound(kycur - f * ay[j]));
-                else
-                  moveTo(qRound(kxcur + f * ax[j]), qRound(kycur - f * ay[j]));
-              else
-                ap[l] = QPoint(qRound(kxcur + f * ax[j]), qRound(kycur - f * ay[j]));
-            }
-            if (k >= 12)
-              drawPolygon(ap, false, 0, l);
-        }
+      if (Utils::valid(xa, logx) && Utils::valid(ya, logy)) {
+        r2i(logx ? log10(xa) : xa, logy ? log10(ya) : ya, &kx, &ky);
+        ap[l++] = QPoint(kx, ky);
       }
     }
-    if ((k > 10) || (k == 1)) {
-      setBrush(NoBrush);
-      setPen(pen3);
+    if (l > 2)
+      drawPolygon(ap, true, 0, l);
+    setBrush(NoBrush);
+    setPen(pen3);
+  } else {
+    if (kSymb > 0) {
+      linetype(kSymb);
+      bool b = false;
+      for (int i = 0; i < n; i++) {
+        xa = fx * x[i];
+        ya = fy * y[i];
+        if (Utils::valid(xa, logx) && Utils::valid(ya, logy))
+          if (b)
+            draw(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
+          else {
+            move(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
+            b = true;
+          }
+      }
+      linetype(1);
+    } else {
+      QPointArray ap(10);
+      int k = (-kSymb) % 17;
+      if ((k > 10) || (k == 1)) {
+        setBrush(pen3.color());
+        setPen(NoPen);
+      }
+      const int ax[48] = {0, -2, 0, 2, 0, 2, -2, -2, 2, 2, 0, -2, 2, 0,
+                          2, -2, 2, -2, 2, 2, 1, -1, -2, -2, -1, 1, 2, 2,
+                          2, -2, 0, -2, 2, 0, 0, 0, -2, 2, 2, 0, -2, 0, 0,
+                          0, -2, 2, 0, 0},
+                ay[48] = {2, 0, -2, 0, 2, 2, 2, -2, -2, 2, 2, -1, -1, 2,
+                          2, 2, -2, -2, 2, 1, 2, 2, 1, -1, -2, -2, -1, 1,
+                          2, -2, 0, 2, -2, 2, -2, 0, 0, 0, 2, 0, 2, 0, -2,
+                          2, 0, 0, 2, -2},
+                as[10] = {0, 5, 10, 14, 19, 28, 33, 38, 43, 48};
+      int j, l = 0, r;
+      double f = 0.005 * relSiz * (maxx - minx);
+      for (int i = 0; i < n; i++) {
+        xa = fx * x[i];
+        ya = fy * y[i];
+        if (Utils::valid(xa, logx) && Utils::valid(ya, logy)) {
+          move(logx ? log10(xa) : xa, logy ? log10(ya) : ya);
+          switch (k) {
+            case 0:
+            case 11:
+              r = qRound(f + f);
+              drawEllipse(kxcur - r, kycur - r, r + r, r + r);
+              break;
+            case 1:
+              r = qRound(0.4 * f);
+              drawEllipse(kxcur - r, kycur - r, r + r, r + r);
+              break;
+            default:
+              r = (k < 12) ? k : k - 10;
+              for (j = as[r - 2]; j < as[r - 1]; j++) {
+                l = j - as[r - 2];
+                if (k < 12)
+                  if (l)
+                    lineTo(qRound(kxcur + f * ax[j]), qRound(kycur - f * ay[j]));
+                  else
+                    moveTo(qRound(kxcur + f * ax[j]), qRound(kycur - f * ay[j]));
+                else
+                  ap[l] = QPoint(qRound(kxcur + f * ax[j]), qRound(kycur - f * ay[j]));
+              }
+              if (k >= 12)
+                drawPolygon(ap, false, 0, l);
+          }
+        }
+      }
+      if ((k > 10) || (k == 1)) {
+        setBrush(NoBrush);
+        setPen(pen3);
+      }
     }
   }
 #ifndef NO_CLIPPING
@@ -922,18 +1025,18 @@ void KGraph::plArray(const double *x, const double *y, double fx, double fy,
 }
 
 void KGraph::plError(const double* x, const double* y, const double* e,
-                     double fx, double fy, int n)
+                     double fx, double fy, int n, bool clip)
 {
 #ifndef NO_CLIPPING
-  setClipping(true);
+  setClipping(clip);
 #endif
-  int idy = qRound(0.01 * relSize * (maxx - minx));
+  int idy = qRound(0.01 * relSiz * (maxx - minx));
   int idx = idy >> 1;
   for (int i = 0; i < n; i++)
-    if (((!logx) || (x[i] > 0)) && ((!logy) || (y[i] > 0))) {
+    if (Utils::valid(x[i], logx) && Utils::valid(y[i], logy)) {
       int ie;
       if (logy)
-        ie = int(-0.4343 * e[i] * yscal / y[i]);
+        ie = int(-0.4342944819 * e[i] * yscal / y[i]);
       else
         ie = int(-fy * e[i] * yscal);
       double xa = fx * x[i];
@@ -943,7 +1046,8 @@ void KGraph::plError(const double* x, const double* y, const double* e,
       if (logy)
         ya = log10(ya);
       if ((xa >= xmin) && (xa <= xmax) &&
-          (ya >= ymin) && (ya <= ymax) && (ie >= idy)) {
+          ((ya - ie * yscal) >= ymin) && ((ya + ie * yscal) <= ymax) &&
+          (ie >= idy)) {
         move(xa, ya);
         moveTo(kxcur, kycur + idy);
         lineTo(kxcur, kycur + ie);
@@ -971,7 +1075,7 @@ void KGraph::linetype(int is)
                          {8, 1}, {7, 2}, {4, 4}, {1, 1}};
 
   int j = QMAX(0, (is - 1) % 9);
-  double f = relSize * (maxx - minx) / (scalx * 320.0);
+  double f = relSiz * (maxx - minx) / (scalx * 320.0);
   double dsp = (double) ((16 - pat[j][0] - pat[j][1]) >> 1);
   dash[0] = f * pat[j][0];
   dash[1] = dash[0] + f * dsp;
@@ -984,7 +1088,7 @@ void KGraph::letxy(const QString& str, int iex, int iax, bool opposite)
   if (str.length() || iex) {
     double x, y;
     QString label;
-    double dx = 0.025 * relSize * (xmax - xmin);
+    double dx = 0.025 * relSiz * (xmax - xmin);
     double dirsav = textdir;
     if (iax == 2) {
       int l = 3;
