@@ -7,7 +7,7 @@
 
     begin                : Sat Apr 24 15:14:00 MEST 1999
 
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -34,6 +34,7 @@
 #include <kedittoolbar.h>
 #include <kfiledialog.h>
 #include <kiconloader.h>
+#include <kimageio.h>
 #include <kkeydialog.h>
 #include <klocale.h>
 #include <kmenubar.h>
@@ -42,7 +43,7 @@
 #include <kstatusbar.h>
 #include <kstdaction.h>
 #include <kstddirs.h>
-#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 1)
+#if KDE_VERSION >= 220
 #include <kprinter.h>
 #else
 #include <qprinter.h>
@@ -56,6 +57,7 @@
 #include "fitdlg.h"
 #include "frameitem.h"
 #include "funitem.h"
+#include "imageitem.h"
 #include "itemdlg.h"
 #include "kpl.h"
 #include "kpldoc.h"
@@ -64,6 +66,7 @@
 #include "lineitem.h"
 #include "psgraph.h"
 #include "refreshdlg.h"
+#include "scalebaritem.h"
 #include "splfitdlg.h"
 #include "splineitem.h"
 #include "textitem.h"
@@ -71,12 +74,14 @@
 #include "utils.h"
 #include "zoomdlg.h"
 
-KplApp::KplApp() : KMainWindow(0, "Kpl"), DCOPObject("KplIface"), itemDlg(0),
+KplApp::KplApp() : KMainWindow(0, "Kpl"), DCOPObject("KplIface"),
   config(kapp->config()), plot(false), tRefresh(5)
 {
+  KImageIO::registerFormats();
   setMinimumSize(120, 80);
   initDocument();
   initActions();
+  itemDlg = new ItemDlg(doc);
   readOptions();
   initView();
   initKeyAccel();
@@ -93,10 +98,14 @@ KplApp::KplApp() : KMainWindow(0, "Kpl"), DCOPObject("KplIface"), itemDlg(0),
   connect(view, SIGNAL(rightButtonPressed()), SLOT(showPopupMenu()));
   connect(view, SIGNAL(urlsDropped(KURL::List*)),
           SLOT(slotOpenList(KURL::List*)));
+  connect(itemDlg, SIGNAL(displayMessage(const QString&)),
+          SLOT(slotStatusMsg(const QString&)));
 }
 
 KplApp::~KplApp()
 {
+  for (unsigned i = 0; i < previews.count(); i++)
+    QFile::remove(previews[i]);
 #if KDE_VERSION_MAJOR < 3
   delete key_accel;
 #endif
@@ -108,16 +117,18 @@ KplDoc* KplApp::getDocument() const
   return doc;
 }
 
-void KplApp::openList(KURL::List* list, int type)
+void KplApp::openList(KURL::List* list, int idec, int type)
 {
   statusBar()->message(i18n("Opening file..."));
+  if (idec < 0)
+    idec = doc->aut.iDec;
   for (unsigned i = 0; i < list->count(); i++)
     if ((!i) || doc->aut.addData) {
       if (doc->aut.addData || (warning->isChecked() ? queryClose() : true))
-        doc->openDocument((*list)[i], type);
+        doc->openDocument((*list)[i], idec, (KplDoc::FileType) type);
     } else {
       KplApp* kpl = new KplApp;
-      if (kpl->doc->openDocument((*list)[i], type))
+      if (kpl->doc->openDocument((*list)[i], idec, (KplDoc::FileType) type))
         kpl->show();
       else
         delete kpl;
@@ -129,13 +140,13 @@ void KplApp::openDocumentURL(const KURL& url)
 {
   statusBar()->message(i18n("Opening file..."));
   if (doc->aut.addData || (warning->isChecked() ? queryClose() : true))
-    doc->openDocument(url);
+    doc->openDocument(url, doc->aut.iDec, KplDoc::Unknown);
   slotStatusMsg(i18n("Ready."));
 }
 
 void KplApp::slotOpenList(KURL::List* list)
 {
-  openList(list);
+  openList(list, doc->aut.iDec, 0);
 }
 
 void KplApp::initActions()
@@ -184,6 +195,14 @@ void KplApp::initActions()
   filePSL = new KAction(i18n("&Landscape..."), 0, this, SLOT(slotFilePSL()),
                         actionCollection(), "postscript_l");
   filePSL->setStatusText(i18n("Generates PostScript file (landscape)"));
+  filePreviewP = new KAction(i18n("&Portrait..."), 0, this,
+                             SLOT(slotFilePreviewP()), actionCollection(),
+                             "preview_p");
+  filePreviewP->setStatusText(i18n("Displays PostScript preview (portrait)"));
+  filePreviewL = new KAction(i18n("&Landscape..."), 0, this,
+                             SLOT(slotFilePreviewL()), actionCollection(),
+                             "preview_l");
+  filePreviewL->setStatusText(i18n("Displays PostScript preview (landscape)"));
   filePrint = KStdAction::print(this, SLOT(slotFilePrint()),
                                 actionCollection());
   filePrint->setStatusText(i18n("Prints the current plot"));
@@ -210,7 +229,11 @@ void KplApp::initActions()
   a->setStatusText(i18n("Sets magnification"));
   viewReload = KStdAction::redisplay(this, SLOT(slotViewReload()),
                                      actionCollection());
+#if KDE_VERSION_MAJOR > 2
+  viewReload->setShortcut(KStdAccel::reload());
+#else
   viewReload->setAccel(KStdAccel::reload());
+#endif
   viewReload->setStatusText(i18n("Reloads file"));
   a = new KAction(i18n("Reload &Periodically..."), 0, this,
                   SLOT(slotViewAutoReload()), actionCollection(),
@@ -230,7 +253,7 @@ void KplApp::initActions()
                               SLOT(slotOptionsAddData()),
                               actionCollection(), "add_data");
   addData->setStatusText(i18n("Enables / disables adding of data and plot files"));
-  showSource = new KToggleAction(i18n("S&how Function Source"), 0, this,
+  showSource = new KToggleAction(i18n("Show Function Sou&rce"), 0, this,
                                  SLOT(slotOptionsShowSource()),
                                  actionCollection(), "show_source");
   showSource->setStatusText(i18n("Enables / disables display of "
@@ -239,6 +262,10 @@ void KplApp::initActions()
                                  actionCollection(), "bounding_box");
   boundingBox->setStatusText(i18n("Enables / disables calculation of "
                                   "PS bounding box"));
+  printPS = new KToggleAction(i18n("Print PS Output"), 0,
+                              actionCollection(), "print_ps");
+  printPS->setStatusText(i18n("Enables / disables printing of "
+                              "PostScript output files"));
   saveAbsPath = new KToggleAction(i18n("Save Absol&ute Paths"), 0,
                               actionCollection(), "save_abs_path");
   saveAbsPath->setStatusText(i18n("Saves absolute paths of data "
@@ -389,8 +416,7 @@ void KplApp::initKeyAccel()
               "margin_top");
 #endif
   createGUI();
-  if (!showToolbar->isChecked())
-    toolBar("mainToolBar")->hide();
+  slotOptionsToolBar();
   toolBar("mainToolBar")->setBarPos(toolBarPos);
 }
 
@@ -409,16 +435,13 @@ void KplApp::initView()
 void KplApp::saveOptions(bool saveAll)
 {
   recentFiles->saveEntries(config);
-  if (itemDlg)
-    Utils::saveGeometry(itemDlg, "ItemDialog");
+  Utils::saveGeometry(itemDlg, "ItemDialog");
   config->setGroup(kapp->instanceName());
   if (saveAll) {
     config->writeEntry("ShowToolbar", showToolbar->isChecked());
     config->writeEntry("ShowStatusbar", showStatusbar->isChecked());
-    config->writeEntry("ShowItemDialog", itemDlg ?
-                       itemDlg->isVisible() : false);
-    if (itemDlg)
-      config->writeEntry("AppendItems", itemDlg->appendItems->isChecked());
+    config->writeEntry("ShowItemDialog", itemDlg->isVisible());
+    config->writeEntry("AppendItems", itemDlg->appendItems->isChecked());
     config->writeEntry("ToolBarPos", (int) toolBar("mainToolBar")->barPos());
     config->writeEntry("Directory", doc->currentDir());
     config->writeEntry("Width", width());
@@ -427,6 +450,7 @@ void KplApp::saveOptions(bool saveAll)
     config->writeEntry("AddData", addData->isChecked());
     config->writeEntry("ShowSource", showSource->isChecked());
     config->writeEntry("BoundingBox", boundingBox->isChecked());
+    config->writeEntry("PrintPS", printPS->isChecked());
     config->writeEntry("SaveAbsPath", saveAbsPath->isChecked());
     config->writeEntry("Warning", warning->isChecked());
     config->writeEntry("SaveAtEnd", saveAtEnd->isChecked());
@@ -467,12 +491,14 @@ void KplApp::saveOptions(bool saveAll)
     config->writeEntry("FitEnableParam", s1, ' ');
     config->writeEntry("FitParam", s2, ' ');
     config->writeEntry("ErrorModelParam", s3, ' ');
+    config->writeEntry("ErrorModelArgument", doc->aut.err.errModArg);
     config->writeEntry("FormatX", doc->aut.xf);
     config->writeEntry("FormatY", doc->aut.yf);
     config->writeEntry("ColorFrame", KplGraph::rgbQt1(doc->aut.colFrame));
     config->writeEntry("ColorGrid", KplGraph::rgbQt1(doc->aut.colGrid));
     config->writeEntry("ColorData", KplGraph::rgbQt1(doc->aut.colData));
     config->writeEntry("OutputFormat", QString(QChar(doc->aut.format)));
+    config->writeEntry("DecimalSymbol", doc->aut.iDec);
     config->writeEntry("Precision", doc->aut.prec);
     config->writeEntry("Separator", doc->aut.iSep);
     config->writeEntry("Zoom", view->zoom());
@@ -494,11 +520,9 @@ void KplApp::readOptions()
   showStatusbar->setChecked(config->readBoolEntry("ShowStatusbar", true));
   if (!showStatusbar->isChecked())
     statusBar()->hide();
-  doc->aut.appendItems = config->readBoolEntry("AppendItems", true);
-  if (config->readBoolEntry("ShowItemDialog")) {
+  itemDlg->appendItems->setChecked(config->readBoolEntry("AppendItems", true));
+  if (config->readBoolEntry("ShowItemDialog"))
     slotEditItems();
-    config->setGroup(kapp->instanceName());
-  }
   toolBarPos = (KToolBar::BarPosition) config->readNumEntry("ToolBarPos",
                                                             KToolBar::Top);
   QString kplDir = KGlobal::dirs()->findResourceDir("appdata", "demo1.plo");
@@ -508,6 +532,7 @@ void KplApp::readOptions()
   addData->setChecked(config->readBoolEntry("AddData"));
   autoplot->setChecked(config->readBoolEntry("AutoPlot", true));
   boundingBox->setChecked(config->readBoolEntry("BoundingBox"));
+  printPS->setChecked(config->readBoolEntry("PrintPS"));
   saveAbsPath->setChecked(config->readBoolEntry("SaveAbsPath", true));
   warning->setChecked(config->readBoolEntry("Warning", true));
   saveAtEnd->setChecked(config->readBoolEntry("SaveAtEnd"));
@@ -555,12 +580,14 @@ void KplApp::readOptions()
     doc->aut.err.pErrMod[i] = (i < cnt) ? list[i].toDouble() : 0.0;
   if (!cnt)
     doc->aut.err.pErrMod[0] = 0.01;
+  doc->aut.err.errModArg = config->readNumEntry("ErrorModelArgument", 1);
   doc->aut.xf = config->readDoubleNumEntry("FormatX", 21.0);
   doc->aut.yf = config->readDoubleNumEntry("FormatY", 29.7);
   doc->aut.colFrame = KplGraph::rgbQt(config->readUnsignedNumEntry("ColorFrame"));
   doc->aut.colGrid = KplGraph::rgbQt(config->readUnsignedNumEntry("ColorGrid"));
   doc->aut.colData = KplGraph::rgbQt(config->readUnsignedNumEntry("ColorData"));
   doc->aut.format = *(config->readEntry("OutputFormat", "g").latin1());
+  doc->aut.iDec = config->readNumEntry("DecimalSymbol");
   doc->aut.prec = config->readNumEntry("Precision", 6);
   doc->aut.iSep = config->readNumEntry("Separator");
   fz = config->readDoubleNumEntry("Zoom", 1.0);
@@ -572,29 +599,30 @@ void KplApp::readOptions()
 
 void KplApp::openRead(int type)
 {
-  KURL::List list = KFileDialog::getOpenURLs(doc->currentDir(),
-                                             type == KplDoc::Data ?
-                                             "*.dat *.DAT\n*" :
-                                             "*.plo *.PLO\n*");
+  int idec = doc->aut.iDec;
+  KURL::List list = (type == KplDoc::Data ?
+                     doc->getReadURLs(idec) :
+                     KFileDialog::getOpenURLs(doc->currentDir(),
+                                              "*.plo *.PLO\n*"));
   if (!list.isEmpty())
-    openList(&list, type);
+    openList(&list, idec, type);
 }
 
 bool KplApp::save(bool current)
 {
   bool success = false;
   if ((doc->URL().fileName() != i18n("Untitled")) && current &&
-      (doc->fType == KplDoc::Plot)) {
+      (doc->fileType() == KplDoc::Plot)) {
     success = doc->saveDocument(doc->URL(), doc->items(),
                                 saveAbsPath->isChecked());
     if (success) {
       doc->setModified(false);
-      doc->fType = KplDoc::Plot;
+      doc->setFileType(KplDoc::Plot);
       updWin();
     }
   } else {
     KURL url;
-    if (Utils::getWriteURL(this, url, "*.plo\n*", doc))
+    if (doc->getWriteURL(this, url, "*.plo\n*"))
       success = saveAs(url);
   }
   return success;
@@ -607,7 +635,7 @@ bool KplApp::saveAs(const KURL& url)
   if (success) {
     doc->setURL(url);
     doc->setModified(false);
-    doc->fType = KplDoc::Plot;
+    doc->setFileType(KplDoc::Plot);
     slotUpdate(false, true);
   }
   return success;
@@ -658,14 +686,14 @@ void KplApp::readProperties(KConfig* _cfg)
     bool canRecover;
     QString tempname = kapp->checkRecoverFile(filename, canRecover);
     if (canRecover) {
-      doc->openDocument(tempname, KplDoc::Plot);
+      doc->openDocument(tempname, doc->aut.iDec, KplDoc::Plot);
       QFile::remove(tempname);
       doc->setModified();
       recovered = true;
     }
   } else
     if (!filename.isEmpty()) {
-      doc->openDocument(u);
+      doc->openDocument(u, doc->aut.iDec, KplDoc::Unknown);
       recovered = true;
     }
   if (recovered)
@@ -673,87 +701,98 @@ void KplApp::readProperties(KConfig* _cfg)
   updWin();
 }
 
-bool KplApp::savePSURL(const KURL& url, bool landscape)
+void KplApp::drawPS(PSGraph* g)
 {
-  QString filename = url.isLocalFile() ? url.path() : doc->tmpFile();
-  QFile f(filename);
+  for (KplItem* item = doc->items()->first(); item;
+       item = doc->items()->next())
+    item->draw(g);
+  g->psExit();
+}
+
+bool KplApp::savePSURL(const KURL& url, bool orientation)
+{
+  QFile f((url.isEmpty() || (!url.isLocalFile())) ?
+          doc->tmpFile() : url.path());
   bool success = f.open(IO_WriteOnly);
   if (success) {
     view->QScrollView::viewport()->setCursor(waitCursor);
     PSGraph g;
     QTextStream ts(&f);
-    g.psInit(&ts, landscape ? doc->aut.yf : doc->aut.xf,
-             landscape ? doc->aut.xf : doc->aut.yf);
-    if (landscape)
-      ts << QString::number(qRound(doc->aut.yf * PSGraph::pts))
-         << " 0 translate 90 rotate\n";
-    for (KplItem* item = doc->items()->first(); item;
-         item = doc->items()->next())
-      item->draw(&g);
-    g.psExit();
+    g.psInit(&ts, doc->aut.xf, doc->aut.yf, orientation);
+    drawPS(&g);
     f.close();
     if (boundingBox->isChecked()) {
-      QString tmpPBM = QDir::homeDirPath() + "/kpltmp.pbm";
+      QString tmpPBM = doc->tmpFile();
       QCString s2;
       KProcess kp;
-      kp << "gs" << "-dNOPAUSE" << "-dBATCH" << "-q" << "-sDEVICE=pbmraw"
-         << ("-sOutputFile=" + tmpPBM) << "-r72"
+      kp << "gs" << "-dNOPAUSE" << "-dBATCH" << "-dSAFER" << "-dPARANOIDSAFER"
+         << "-q" << "-sDEVICE=pbmraw" << ("-sOutputFile=" + tmpPBM) << "-r72"
          << s2.sprintf("-g%ix%i", qRound(PSGraph::pts *
-           (landscape ? doc->aut.yf : doc->aut.xf)),
-           qRound(PSGraph::pts * (landscape ? doc->aut.xf : doc->aut.yf)))
-         << filename;
+           (orientation ? doc->aut.yf : doc->aut.xf)),
+           qRound(PSGraph::pts * (orientation ? doc->aut.xf : doc->aut.yf)))
+         << f.name();
       kp.start(KProcess::Block);
       QImage image;
       if (image.load(tmpPBM, "PBMRAW")) {
-        int x, y, xl = 0, yo = 0;
-        int xr = image.width();
-        int yu = image.height();
+        int x, y, llx = 0, ury = 0;
+        int urx = image.width();
+        int lly = image.height();
         for (y = 0; y < image.height(); y++)
           for (x = 0; x < image.width(); x++)
             if (!(image.pixel(x, y) & 0xffffff)) {
-              yo = y;
-              xl = xr = x;
+              ury = y;
+              llx = urx = x;
               goto yo_found;
             }
 yo_found:
-        for (y = image.height() - 1; y > yo; y--)
+        for (y = image.height() - 1; y > ury; y--)
           for (x = 0; x < image.width(); x++)
             if (!(image.pixel(x, y) & 0xffffff)) {
-              yu = y;
-              xl = QMIN(xl, x);
-              xr = QMAX(xr, x);
+              lly = y;
+              llx = QMIN(llx, x);
+              urx = QMAX(urx, x);
               goto yu_found;
             }
 yu_found:
-        for (y = yo; y <= yu; y++){
-          for (x = 0; x < xl; x++)
+        for (y = ury; y <= lly; y++){
+          for (x = 0; x < llx; x++)
             if (!(image.pixel(x, y) & 0xffffff)) {
-              xl = x;
+              llx = x;
               break;
             }
-          for (x = image.width() - 1; x > xr; x--)
+          for (x = image.width() - 1; x > urx; x--)
             if (!(image.pixel(x, y) & 0xffffff)) {
-              xr = x;
+              urx = x;
               break;
             }
         }
-        xl = QMAX(xl - 2, 0);
-        xr = QMIN(xr + 2, image.width() - 1);
-        yu = QMAX(image.height() - yu - 2, 0);
-        yo = QMIN(image.height() - yo + 2, image.height() - 1);
-        remove(tmpPBM);
-        f.open(IO_ReadWrite);
-        ts << "%!PS-Adobe-3.0 EPSF-3.0\n%%BoundingBox: "
-           << QString::number(xl) << " " << QString::number(yu) << " "
-           << QString::number(xr) << " " << QString::number(yo) << "      ";
+        llx = QMAX(llx - 2, 0);
+        urx = QMIN(urx + 2, image.width() - 1);
+        lly = QMAX(image.height() - lly - 2, 0);
+        ury = QMIN(image.height() - ury + 2, image.height() - 1);
+        QFile::remove(tmpPBM);
+        f.open(IO_WriteOnly);
+        g.psInit(&ts, doc->aut.xf, doc->aut.yf, orientation,
+                 llx, lly, urx, ury);
+        drawPS(&g);
         f.close();
       } else
         KMessageBox::error(this,
           i18n("Bitmap file for bounding box calculation not loaded!"));
     }
-    doc->setCurrentDir(url);
-    if (!url.isLocalFile())
-      doc->copyTmp(url);
+    if (url.isEmpty()) {
+      previews << f.name();
+      KApplication::startServiceByDesktopName("kghostview", f.name());
+    } else {
+      doc->setCurrentDir(url);
+      if (printPS->isChecked()) {
+        KProcess kp;
+        kp << "kprinter" << f.name();
+        kp.start(KProcess::Block);
+      }
+      if (!url.isLocalFile())
+        doc->copyTmp(f.name(), url);
+    }
     view->QScrollView::viewport()->setCursor(crossCursor);
   } else
     KMessageBox::sorry(this, i18n("Writing to this file not possible."));
@@ -823,10 +862,14 @@ void KplApp::slotPlotCond()
 void KplApp::slotFilePrint()
 {
   statusBar()->message(i18n("Printing..."));
-#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 1)
+#if KDE_VERSION_MAJOR > 2
+  KPrinter printer(true, QPrinter::HighResolution);
+#else
+#if KDE_VERSION_MINOR > 1
   KPrinter printer;
 #else
   QPrinter printer;
+#endif
 #endif
   if (printer.setup(this))
     view->print(&printer);
@@ -843,16 +886,34 @@ void KplApp::slotFilePSL()
   slotFilePS(true);
 }
 
-void KplApp::slotFilePS(bool landscape)
+void KplApp::slotFilePS(bool orientation)
 {
   statusBar()->message(i18n("Generating PostScript file..."));
   KURL u;
-  if (Utils::getWriteURL(this, u, "*.ps *.eps\n*", doc))
-    if (!savePSURL(u, landscape))
+  if (doc->getWriteURL(this, u, "*.ps *.eps\n*"))
+    if (!savePSURL(u, orientation))
       KMessageBox::sorry(this, i18n("Writing to this file not possible."));
   slotStatusMsg(i18n("Ready."));
 }
 
+void KplApp::slotFilePreviewP()
+{
+  slotFilePreview(false);
+}
+
+void KplApp::slotFilePreviewL()
+{
+  slotFilePreview(true);
+}
+
+void KplApp::slotFilePreview(bool orientation)
+{
+  statusBar()->message(i18n("Generating PostScript preview..."));
+  KURL u;
+  savePSURL(u, orientation);
+  slotStatusMsg(i18n("Ready."));
+}
+
 void KplApp::slotFileNewWindow()
 {
   (new KplApp)->show();
@@ -891,11 +952,6 @@ void KplApp::slotEditRedo()
 
 void KplApp::slotEditItems()
 {
-  if (!itemDlg) {
-    itemDlg = new ItemDlg(doc);
-    connect(itemDlg, SIGNAL(displayMessage(const QString&)),
-            SLOT(slotStatusMsg(const QString&)));
-  }
   itemDlg->show();
   itemDlg->raise();
 }
@@ -942,7 +998,7 @@ void KplApp::slotViewReload()
   autoplot->setChecked(true);
   bool sav = doc->aut.addData;
   doc->aut.addData = false;
-  doc->openDocument(KURL(doc->URL()), doc->fType);
+  doc->openDocument(KURL(doc->URL()), doc->aut.iDec, doc->fileType());
   doc->aut.addData = sav;
   slotStatusMsg(i18n("Ready."));
 }
@@ -966,15 +1022,26 @@ void KplApp::slotViewAutoReload()
 
 void KplApp::slotOptionsToolBar()
 {
-  toolBar("mainToolBar")->enable(KToolBar::Toggle);
+#if QT_VERSION >= 0x030100
+  toolBar("mainToolBar")->setShown(showToolbar->isChecked());
+#else
+  if (showToolbar->isChecked())
+    toolBar("mainToolBar")->show();
+  else
+    toolBar("mainToolBar")->hide();
+#endif
 }
 
 void KplApp::slotOptionsStatusBar()
 {
-  if (statusBar()->isVisible())
-    statusBar()->hide();
-  else
+#if QT_VERSION >= 0x030100
+  statusBar()->setShown(showStatusbar->isChecked());
+#else  
+  if (showStatusbar->isChecked())
     statusBar()->show();
+  else
+    statusBar()->hide();
+#endif
 }
 
 void KplApp::slotOptionsAutoPlot()
@@ -998,7 +1065,11 @@ void KplApp::slotOptionsShowSource()
 void KplApp::slotOptionsKeys()
 {
   statusBar()->message(i18n("Key bindings..."));
+#if KDE_VERSION_MAJOR > 2
+  KKeyDialog::configure(actionCollection());
+#else
   KKeyDialog::configureKeys(actionCollection(), xmlFile());
+#endif
   slotStatusMsg(i18n("Ready."));
 }
 
@@ -1064,7 +1135,7 @@ void KplApp::slotUpdate(bool updPlot, bool updList)
       slotPlotCond();
     else
       view->eraseWidget();
-  if (itemDlg && updList)
+  if (updList)
     itemDlg->updItemList();
 }
 
@@ -1086,6 +1157,8 @@ void KplApp::updWin()
   filePlot->setEnabled(nItems);
   filePSP->setEnabled(nItems);
   filePSL->setEnabled(nItems);
+  filePreviewP->setEnabled(nItems);
+  filePreviewL->setEnabled(nItems);
   filePrint->setEnabled(nItems);
   fileSave->setEnabled(doc->isModified());
   editUndo->setEnabled(doc->undoAllowed());
@@ -1149,12 +1222,12 @@ void KplApp::newPlot()
 
 int KplApp::openPlot(const QString& url)
 {
-  return doc->openDocument(KURL(url), KplDoc::Plot) ? 1 : 0;
+  return doc->openDocument(KURL(url), 0, KplDoc::Plot) ? 1 : 0;
 }
 
 int KplApp::openData(const QString& url)
 {
-  return doc->openDocument(KURL(url), KplDoc::Data) ? 1 : 0;
+  return doc->openDocument(KURL(url), doc->aut.iDec, KplDoc::Data) ? 1 : 0;
 }
 
 void KplApp::doPlot()
@@ -1170,29 +1243,29 @@ int KplApp::savePlot(const QString& url)
     return 0;
 }
 
-int KplApp::savePS(const QString& url, int landscape)
+int KplApp::savePS(const QString& url, int orientation)
 {
   if (doc->items()->count())
-    return savePSURL(KURL(url), landscape) ? 1 : 0;
+    return savePSURL(KURL(url), orientation) ? 1 : 0;
   else
     return 0;
 }
 
 int KplApp::printPlot(const QString& printerName, const QString& fileName,
-                      int landscape, int numCopies)
+                      int orientation, int numCopies)
 {
   if (doc->items()->count()) {
-#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 1)
+#if KDE_VERSION >= 220
     KPrinter p;
 #else
     QPrinter p;
 #endif
     p.setPrinterName(printerName);
     p.setOutputFileName(fileName);
-#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 1)
-    p.setOrientation(landscape ? KPrinter::Landscape : KPrinter::Portrait);
+#if KDE_VERSION >= 220
+    p.setOrientation(orientation ? KPrinter::Landscape : KPrinter::Portrait);
 #else
-    p.setOrientation(landscape ? QPrinter::Landscape : QPrinter::Portrait);
+    p.setOrientation(orientation ? QPrinter::Landscape : QPrinter::Portrait);
 #endif
     p.setNumCopies(numCopies);
     view->print(&p);
@@ -1302,7 +1375,7 @@ void KplApp::setAutofit(const QString& path, const QString& _name,
 
 int KplApp::loadAutofitPar(const QString& url)
 {
-  return Utils::loadPar(KURL(url), this, doc->aut.pFit, doc) ? 1 : 0;
+  return doc->loadPar(KURL(url), this, doc->aut.pFit, doc->aut.iDec) ? 1 : 0;
 }
 
 int KplApp::setAutofitPar(int iPar, double value)
@@ -1327,7 +1400,7 @@ int KplApp::loadErrModPar(const QString& url, int iArr)
   if (iArr && (unsigned(iArr) >= arr.count()))
     return success;
   double p[KPL_NPMAX];
-  success = Utils::loadPar(KURL(url), this, p, doc) ? 1 : 0;
+  success = doc->loadPar(KURL(url), this, p, doc->aut.iDec) ? 1 : 0;
   if (success) {
     QString s;
     s.sprintf("DataErrors%i", iArr);
@@ -1427,7 +1500,7 @@ int KplApp::newArrayItem(const QString& path, int ix, int iy, int ie,
                          const QString& color, double fx, double fy,
                          int active)
 {
-  doc->items()->append(new ArrayItem(active, symb, color, fx, fy,
+  doc->items()->append(new ArrayItem(active, 0, symb, color, fx, fy,
                                      ix, iy, ie, istart, n, errbars, path));
   ArrayItem* it = (ArrayItem*) doc->items()->current();
   if (it->nrows) {
@@ -1445,7 +1518,7 @@ int KplApp::newFunItem(const QString& path, const QString& _name,
                        int symb, const QString& color,
                        double fx, double fy, int active)
 {
-  doc->items()->append(new FunItem(active, symb, color, fx, fy,
+  doc->items()->append(new FunItem(active, 0, symb, color, fx, fy,
                                    xmin, xmax, dx, _name, path));
   FunItem* it = (FunItem*) doc->items()->current();
   if (it->hmody) {
@@ -1464,8 +1537,8 @@ int KplApp::newParFunItem(const QString& pathx, const QString& namex,
                           int symb, const QString& color,
                           double fx, double fy, int active)
 {
-  doc->items()->append(new ParFunItem(active, symb, color, fx, fy, tmin, tmax,
-                                      dt, namex, pathx, namey, pathy));
+  doc->items()->append(new ParFunItem(active, 0, symb, color, fx, fy, tmin,
+                                      tmax, dt, namex, pathx, namey, pathy));
   ParFunItem* it = (ParFunItem*) doc->items()->current();
   if (it->hmodx || it->hmody) {
     doc->setModified();
@@ -1481,8 +1554,8 @@ void KplApp::newSplineItem(int deriv, double low, double xmin, double xmax,
                            double dx, int symb, const QString& color,
                            double relSize, double fx, double fy, int active)
 {
-  doc->items()->append(new SplineItem(active, symb, color, fx, fy, xmin, xmax,
-                                      dx, deriv, low, relSize));
+  doc->items()->append(new SplineItem(active, 0, symb, color, fx, fy, xmin,
+                                      xmax, dx, deriv, low, relSize));
   doc->setModified();
   doc->backupItems();
 }
@@ -1561,10 +1634,18 @@ int KplApp::newFun3DItem(double x0, double w, double y0, double h,
   }
 }
 
-void KplApp::newLegendItem(int symb, const QString& colSymb, double x, double y,
-                           const QString& text, const QString& colText,
-                           double xoff, double yoff, double relSize,
-                           int active)
+void KplApp::newImageItem(const QString& path, double x, double y, double fx,
+                          double fy, int active)
+{
+  doc->items()->append(new ImageItem(active, x, y, fx, fy, path));
+  doc->setModified();
+  doc->backupItems();
+}
+
+void KplApp::newLegendItem(int symb, const QString& colSymb, double x,
+                           double y, const QString& text,
+                           const QString& colText, double xoff, double yoff,
+                           double relSize, int active)
 {
   doc->items()->append(new LegendItem(active, symb, colSymb, colText, x, y,
                                       xoff, yoff, relSize, text));
@@ -1628,6 +1709,16 @@ void KplApp::newEllipseItem(const QString& color, double x, double y,
   doc->backupItems();
 }
 
+void KplApp::newScaleBarItem(const QString& str, const QString& color,
+                             double x, double y, double len, double relSize,
+                             int o, int active)
+{
+  doc->items()->append(new ScaleBarItem(active, color, str, x, y, o, len,
+                                        relSize));
+  doc->setModified();
+  doc->backupItems();
+}
+
 int KplApp::autoScaleFrame(int iItem, int autoNorm)
 {
   if (unsigned(iItem) >= doc->items()->count())
@@ -1680,7 +1771,7 @@ int KplApp::loadPar(const QString& url, int iItem, int yFun)
   else
     p = yFun ? ((ParFunItem*) doc->items()->at(iItem))->py :
                ((ParFunItem*) doc->items()->at(iItem))->px;
-  success = Utils::loadPar(KURL(url), this, p, doc) ? 1 : 0;
+  success = doc->loadPar(KURL(url), this, p, doc->aut.iDec) ? 1 : 0;
   if (success) {
     doc->setModified();
     doc->backupItems();
@@ -1800,8 +1891,7 @@ double KplApp::chiSquare()
 
 int KplApp::saveFitPar(const QString& url)
 {
-  return Utils::saveParameter(this, KURL(url), doc->pLast,
-                              doc->pErr, doc) ? 1 : 0;
+  return doc->saveParameter(this, KURL(url), doc->pLast, doc->pErr) ? 1 : 0;
 }
 
 int KplApp::exportValues(const QString& url, int iItem)
@@ -1818,7 +1908,7 @@ int KplApp::exportValues(const QString& url, int iItem)
     doc->items()->at(iItem)->exportTable(ts, doc);
     f.close();
     if (!u.isLocalFile())
-      doc->copyTmp(u);
+      doc->copyTmp(f.name(), u);
   } else
     KMessageBox::sorry(this, i18n("Writing to this file not possible."));
   return success;
