@@ -7,7 +7,7 @@
 
     begin                : Sat Apr 24 15:14:00 MEST 1999
 
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -42,6 +42,7 @@ class FunItem;
 class ItemDlg;
 class KplDoc;
 class KplView;
+class PSGraph;
 
 /**
   * Kpl application.
@@ -67,9 +68,10 @@ public:
   /**
    * Opens files from URL list.
    * @param list pointer to list with URLs.
+   * @param idec 0 for decimal point, 1 for comma, -1 for auto settings.
    * @param type one of the following: Unknown, Data, Plot.
    */
-  void openList(KURL::List* list, int type = 0);
+  void openList(KURL::List* list, int idec, int type);
 
 k_dcop:
   /** Creates a new plot in the current view. */
@@ -97,20 +99,20 @@ k_dcop:
   /**
    * Generates PostScript file.
    * @param url URL of the file.
-   * @param landscape true for landscape orientation.
+   * @param orientation true for landscape orientation.
    * @return true when when succeeded.
    */
-  int savePS(const QString& url, int landscape);
+  int savePS(const QString& url, int orientation);
   /**
    * Prints plot.
    * @param printerName name of the printer.
    * @param fileName name of the output file (empty for output to printer).
-   * @param landscape true for landscape orientation.
+   * @param orientation true for landscape orientation.
    * @param numCopies number of copies.
    * @return true when when succeeded.
    */
   int printPlot(const QString& printerName, const QString & fileName,
-                int landscape, int numCopies);
+                int orientation, int numCopies);
   /**
    * Sets autoplot option.
    * @param setting true to plot automatically after changes.
@@ -407,7 +409,7 @@ k_dcop:
    * @param frame true to show frame.
    * @param gridmode 0 for frame only, 1 for axes with tics,
    * 2 for axes with tics and labels.
-   * @param gridmode3 0 for lines parallel to x, 1 for lines parallel to z,
+   * @param gridmode3D 0 for lines parallel to x, 1 for lines parallel to z,
    * 2 for x,z grid.
    * @param xBox x box width in cm.
    * @param yBox y box width in cm.
@@ -488,7 +490,7 @@ k_dcop:
    * @param frame true to show frame.
    * @param gridmode 0 for frame only, 1 for axes with tics,
    * 2 for axes with tics and labels.
-   * @param gridmode3 0 for lines parallel to x, 1 for lines parallel to z,
+   * @param gridmode3D 0 for lines parallel to x, 1 for lines parallel to z,
    * 2 for x,z grid.
    * @param xBox x box width in cm.
    * @param yBox y box width in cm.
@@ -527,9 +529,20 @@ k_dcop:
                    double phi, double theta, int iex, int iey, int iez,
                    const QString& sx, const QString& sy,
                    const QString& sz, const QString& sh, double relSize,
-                   const QString& path, const QString& _name,
+                   const QString& path, const QString& name,
                    double dx, double dz, double fx, double fy,
                    double fz, int active);
+  /**
+   * Appends new image item.
+   * @param path absolute path of the EPS file.
+   * @param x x position in cm.
+   * @param y y position in cm.
+   * @param fx x scaling factor.
+   * @param fy y scaling factor.
+   * @param active true for visible item.
+   */
+  void newImageItem(const QString& path, double x, double y, double fx,
+                    double fy, int active);
   /**
    * Appends new legend item.
    * @param symb 0 for circles, < 0 for different markers,
@@ -550,10 +563,9 @@ k_dcop:
                      int active);
   /**
    * Appends new text item.
-   * Constructor. Initializes corresponding to arguments.
    * @param text text.
    * @param color color for text.
-   * @param iAlign 1: left aligned, 2: centered, 3: right aligned.
+   * @param align 1: left aligned, 2: centered, 3: right aligned.
    * @param x x start position in cm.
    * @param y y start position in cm.
    * @param dir direction in degree.
@@ -565,7 +577,6 @@ k_dcop:
                    int active);
   /**
    * Appends new line item.
-   * Constructor. Initializes corresponding to arguments.
    * @param symb 1 to 9 for different line types.
    * @param color color for line.
    * @param x x start position in cm.
@@ -579,7 +590,6 @@ k_dcop:
                    double dir, double len, double relSize, int active);
   /**
    * Appends new arrow item.
-   * Constructor. Initializes corresponding to arguments.
    * @param color color for arrow.
    * @param x x position of head in cm.
    * @param y y position of head in cm.
@@ -591,7 +601,7 @@ k_dcop:
   void newArrowItem(const QString& color, double x, double y,
                     double dir, double len, double relSize, int active);
   /**
-    * Constructor. Initializes corresponding to arguments.
+    * Appends new arc item.
     * @param color color for arc.
     * @param x x position in cm.
     * @param y y position in cm.
@@ -608,34 +618,46 @@ k_dcop:
                   int active);
   /**
    * Appends new rectangle item.
-   * Constructor. Initializes corresponding to arguments.
    * @param color color for rectangle.
    * @param x x position of left side in cm.
    * @param y y position of bottom side in cm.
    * @param w width in cm.
    * @param h height in cm.
    * @param relSize relative line width.
-   * @param fill true for filled with color.
+   * @param fillStyle > 0 for pattern filled rectangle.
    * @param active true for visible item.
    */
   void newRectangleItem(const QString& color, double x, double y,
-                        double w, double h, double relSize, int fill,
+                        double w, double h, double relSize, int fillStyle,
                         int active);
   /**
    * Appends new ellipse item.
-   * Constructor. Initializes corresponding to arguments.
    * @param color color for ellipse.
    * @param x x position of center in cm.
    * @param y y position of center in cm.
    * @param w width in cm.
    * @param h height in cm.
    * @param relSize relative line width.
-   * @param fill true for filled with color.
+   * @param fillStyle > 0 for pattern filled ellipse.
    * @param active true for visible item.
    */
   void newEllipseItem(const QString& color, double x, double y,
-                      double w, double h, double relSize, int fill,
+                      double w, double h, double relSize, int fillStyle,
                       int active);
+  /**
+   * Appends new scale bar item.
+   * @param str text.
+   * @param color color for scale bar.
+   * @param x x position of bar in cm.
+   * @param y y position of bar in cm.
+   * @param len width in cm.
+   * @param relSize relative line width.
+   * @param orientation 0 for horizontal, 1 for vertical.
+   * @param active true for visible item.
+   */
+  void newScaleBarItem(const QString& str, const QString& color,
+                       double x, double y, double len, double relSize,
+                       int orientation, int active);
   /**
    * Scales automatically all child items of a frame item.
    * @param iItem item index.
@@ -665,7 +687,7 @@ k_dcop:
   /**
    * Sets array row range.
    * @param iItem item index.
-   * @param istart start row index.
+   * @param iStart start row index.
    * @param n number of data points.
    * @return true when when succeeded.
    */
@@ -776,13 +798,18 @@ protected:
    * @return true when file is saved.
    */
   bool saveAs(const KURL &url);
+  /**
+   * Draws items for PostScript file.
+   * @param g pointer to PSGraph instance.
+   */
+  void drawPS(PSGraph* g);
   /**
    * Generates PostScript file.
    * @param url URL of the file.
-   * @param landscape != 0 for landscape orientation.
+   * @param orientation != 0 for landscape orientation.
    * @return true when file is saved.
    */
-  bool savePSURL(const KURL& url, bool landscape);
+  bool savePSURL(const KURL& url, bool orientation);
   /**
    * Warns at unsaved changes.
    * @return true when ready to close file.
@@ -811,6 +838,8 @@ protected:
   KAction* filePlot;
   KAction* filePSP;
   KAction* filePSL;
+  KAction* filePreviewP;
+  KAction* filePreviewL;
   KAction* editUndo;
   KAction* editRedo;
   KAction* viewReload;
@@ -820,6 +849,7 @@ protected:
   KToggleAction* addData;
   KToggleAction* showSource;
   KToggleAction* boundingBox;
+  KToggleAction* printPS;
   KToggleAction* saveAbsPath;
   KToggleAction* warning;
   KToggleAction* saveAtEnd;
@@ -833,6 +863,7 @@ protected:
   KToolBar::BarPosition toolBarPos;
   QList<ArrayItem> arr;
   QList<FunItem> fun;
+  QStringList previews;
   enum StatusId{StatusMsg = 1, StatusAdd, StatusPos, StatusVal};
   bool plot;
   int tRefresh, xPrec, yPrec;
@@ -875,9 +906,18 @@ protected slots:
   void slotFilePSL();
   /**
    * Generates PostScript file.
-   * @param landscape true for landscape orientation.
+   * @param orientation true for landscape orientation.
    */
-  void slotFilePS(bool landscape);
+  void slotFilePS(bool orientation);
+  /** Displays PostScript preview (portrait orientation). */
+  void slotFilePreviewP();
+  /** Displays PostScript preview (landscape orientation). */
+  void slotFilePreviewL();
+  /**
+   * Displays PostScript preview.
+   * @param orientation true for landscape orientation.
+   */
+  void slotFilePreview(bool orientation);
   /** Prints the current file. */
   void slotFilePrint();
   /** Opens a new kpl window. */
