@@ -12,7 +12,6 @@
 #include <kdatastream.h>
 
 
-
 KplApp_stub::KplApp_stub( const QCString& app, const QCString& obj )
   : DCOPStub( app, obj )
 {
@@ -922,6 +921,28 @@ int KplApp_stub::newFun3DItem( double arg0, double arg1, double arg2, double arg
     return result;
 }
 
+void KplApp_stub::newImageItem( const QString& arg0, double arg1, double arg2, double arg3, double arg4, int arg5 )
+{
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    if ( dcopClient()->call( app(), obj(), "newImageItem(QString,double,double,double,double,int)", data, replyType, replyData ) ) {
+	setStatus( CallSucceeded );
+    } else { 
+	callFailed();
+    }
+}
+
 void KplApp_stub::newLegendItem( int arg0, const QString& arg1, double arg2, double arg3, const QString& arg4, const QString& arg5, double arg6, double arg7, double arg8, int arg9 )
 {
     if ( !dcopClient()  ) {
@@ -1093,6 +1114,30 @@ void KplApp_stub::newEllipseItem( const QString& arg0, double arg1, double arg2,
     }
 }
 
+void KplApp_stub::newScaleBarItem( const QString& arg0, const QString& arg1, double arg2, double arg3, double arg4, double arg5, int arg6, int arg7 )
+{
+    if ( !dcopClient()  ) {
+	setStatus( CallFailed );
+	return;
+    }
+    QByteArray data, replyData;
+    QCString replyType;
+    QDataStream arg( data, IO_WriteOnly );
+    arg << arg0;
+    arg << arg1;
+    arg << arg2;
+    arg << arg3;
+    arg << arg4;
+    arg << arg5;
+    arg << arg6;
+    arg << arg7;
+    if ( dcopClient()->call( app(), obj(), "newScaleBarItem(QString,QString,double,double,double,double,int,int)", data, replyType, replyData ) ) {
+	setStatus( CallSucceeded );
+    } else { 
+	callFailed();
+    }
+}
+
 int KplApp_stub::autoScaleFrame( int arg0, int arg1 )
 {
     int result = 0;
