@@ -7,7 +7,7 @@
 
     begin                : Sat Apr 24 15:14:00 MEST 1999
 
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -20,32 +20,40 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <stdlib.h>
 #include <math.h>
+#include <stdlib.h>
+#include <qcombobox.h>
 #include <qdir.h>
-#include <klocale.h>
+#include <qhbox.h>
+#include <qlabel.h>
+#include <qlineedit.h>
+#include <kapp.h>
+#include <kfiledialog.h>
 #include <kglobal.h>
-#include <ksimpleconfig.h>
-#include <kmessagebox.h>
 #include <kio/netaccess.h>
-#include "kpldoc.h"
+#include <klocale.h>
+#include <kmessagebox.h>
+#include <ksimpleconfig.h>
+#include <ktempfile.h>
+#include "arcitem.h"
+#include "arrayitem.h"
+#include "ellipseitem.h"
 #include "fitdlg.h"
 #include "frameitem.h"
-#include "arrayitem.h"
 #include "funitem.h"
-#include "splineitem.h"
-#include "threeditem.h"
+#include "imageitem.h"
+#include "kpldoc.h"
 #include "legenditem.h"
-#include "textitem.h"
 #include "lineitem.h"
-#include "arcitem.h"
 #include "rectitem.h"
-#include "ellipseitem.h"
+#include "scalebaritem.h"
+#include "splineitem.h"
+#include "textitem.h"
+#include "threeditem.h"
 #include "utils.h"
 
 KplDoc::KplDoc(QObject* _parent) :
- QObject(_parent), modified(false), fType(Unknown),
- m_tmpfile(QDir::homeDirPath() + "/kpltmp.tmp"), job(0), chisq(0.0)
+ QObject(_parent), modified(false), fType(Unknown), chisq(0.0)
 {
   memset(pLast, 0, sizeof(pLast));
   memset(pErr, 0, sizeof(pErr));
@@ -84,7 +92,7 @@ bool KplDoc::saveDocument(const KURL& url, QList<KplItem>* _items, bool _abs)
       }
     }
     if (!url.isLocalFile())
-      copyTmp(url);
+      copyTmp(filename, url);
   } else
     KMessageBox::sorry((QWidget*) parent(),
                        i18n("Writing to this file not possible."));
@@ -135,6 +143,16 @@ const QDateTime& KplDoc::URLTime() const
   return m_time;
 }
 
+KplDoc::FileType KplDoc::fileType() const
+{
+  return fType;
+}
+
+void KplDoc::setFileType(FileType iType)
+{
+  fType = iType;
+}
+
 void KplDoc::freeAllItems()
 {
   itd.clear();
@@ -186,6 +204,12 @@ void KplDoc::newItem(KplItem::ItemTypes ityp, int index)
       break;
     case KplItem::Ellipse:
       it = new EllipseItem(&aut);
+      break;
+    case KplItem::Image:
+      it = new ImageItem(&aut);
+      break;
+    case KplItem::ScaleBar:
+      it = new ScaleBarItem(&aut);
   }
   if (index < 0)
     itd.append(it);
@@ -193,7 +217,7 @@ void KplDoc::newItem(KplItem::ItemTypes ityp, int index)
     itd.insert(index, it);
 }
 
-bool KplDoc::autoFile(double* xmi, double* xma, double* tic, int* mtic,
+void KplDoc::autoFile(double* xmi, double* xma, double* tic, int* mtic,
                       int* ndig, int* ie, double* fn, const double* x, int n,
                       double c, bool _log)
 {
@@ -205,20 +229,18 @@ bool KplDoc::autoFile(double* xmi, double* xma, double* tic, int* mtic,
     *ie = 0;
     *fn = 1.0;
   }
-  return Utils::autoSc(xmi, xma, tic, mtic, ndig, xmin, xmax, *fn, c, _log);
+  Utils::autoSc(xmi, xma, tic, mtic, ndig, xmin, xmax, *fn, c, _log);
 }
 
 bool KplDoc::autoScale(FrameItem* fd, ArrayItem* ad)
 {
   if ((ad->x) && (aut.ixAuto < ad->ncols) && (aut.iyAuto < ad->ncols)) {
-    if (autoFile(&fd->xmi, &fd->xma, &fd->xtic, &fd->mticx, &fd->ndigx,
-                 &fd->iex, &ad->fx, ad->x[aut.ixAuto], ad->nrows,
-                 0.2, aut.autoLogX))
-      return true;
-    if (autoFile(&fd->ymi, &fd->yma, &fd->ytic, &fd->mticy, &fd->ndigy,
-                 &fd->iey, &ad->fy, ad->x[aut.iyAuto], ad->nrows,
-                 0.2 * aut.xlAuto / aut.ylAuto, aut.autoLogY))
-      return true;
+    autoFile(&fd->xmi, &fd->xma, &fd->xtic, &fd->mticx, &fd->ndigx,
+             &fd->iex, &ad->fx, ad->x[aut.ixAuto], ad->nrows,
+             0.2, aut.autoLogX);
+    autoFile(&fd->ymi, &fd->yma, &fd->ytic, &fd->mticy, &fd->ndigy,
+             &fd->iey, &ad->fy, ad->x[aut.iyAuto], ad->nrows,
+             0.2 * aut.xlAuto / aut.ylAuto, aut.autoLogY);
     if (aut.autohPath)
       fd->sh = ad->url.isLocalFile() ? ad->url.path() : ad->url.url();
     int minCol = ad->ncols - 1;
@@ -310,15 +332,16 @@ QList<KplItem>* KplDoc::items()
   return &itd;
 }
 
-const QString& KplDoc::tmpFile() const
+QString KplDoc::tmpFile() const
 {
-  return m_tmpfile;
+  KTempFile f;
+  return f.name();
 }
 
-void KplDoc::copyTmp(const KURL& dest)
+void KplDoc::copyTmp(const QString& src, const KURL& dest)
 {
-  connect(job = KIO::file_copy(KURL(tmpFile()), dest, -1, true),
-          SIGNAL(result(KIO::Job*)), this, SLOT(slotIOfinished()));
+  connect(KIO::file_copy(KURL(src), dest, -1, true),
+          SIGNAL(result(KIO::Job*)), this, SLOT(slotIOfinished(KIO::Job*)));
 }
 
 const QString KplDoc::number(double val) const
@@ -326,6 +349,146 @@ const QString KplDoc::number(double val) const
   return QString::number(val, aut.format, aut.prec);
 }
 
+bool KplDoc::getWriteURL(QWidget* parent, KURL& url, const QString& filter)
+{
+  url = KFileDialog::getSaveURL(currentDir(), filter);
+  if (url.isEmpty())
+    return false;
+  if (url.isLocalFile()) {
+    QFileInfo fi(url.path());
+    if (!fi.exists())
+      return true;
+    if (!fi.isWritable()) {
+      KMessageBox::sorry(parent, i18n("Writing to this file not possible."));
+      return false;
+    }
+  } else {
+#if KDE_VERSION >= 0x03015a
+    if (!KIO::NetAccess::exists(url, true, parent))
+      return true;
+    if (!KIO::NetAccess::exists(url, false, parent)) {
+      KMessageBox::sorry(parent, i18n("Writing to this file not possible."));
+      return false;
+    }
+#else
+    if (!KIO::NetAccess::exists(url))
+      return true;
+#endif
+  }
+  return (KMessageBox::warningYesNo(parent, url.path() + "\n" +
+    i18n("A file with this name exists already.\n"
+         "Do you want to overwrite it?")) == KMessageBox::Yes);
+}
+
+#if KDE_VERSION >= 0x030100
+KURL::List KplDoc::getReadURLs(int& idec)
+{
+  QHBox* hbox = new QHBox();
+  KFileDialog dlg(currentDir(), "*.dat *.DAT\n*", 0, 0, true, hbox);
+  QComboBox* cb = decimalWidget(idec, &dlg, hbox, KFile::Files);
+  dlg.exec();
+  idec = cb->currentItem();
+#else
+KURL::List KplDoc::getReadURLs(int&)
+{
+  KFileDialog dlg(currentDir(), "*.dat *.DAT\n*", 0, 0, true);
+  dlg.exec();
+#endif
+  return dlg.selectedURLs();
+}
+
+KURL KplDoc::getReadURL(int& idec, const QString& filename,
+                        const QString& filter)
+{
+#if KDE_VERSION >= 0x030100
+  QHBox* hbox = new QHBox();
+  KFileDialog dlg(currentDir() + filename, filter, 0, 0, true, hbox);
+  QComboBox* cb = decimalWidget(idec, &dlg, hbox, KFile::File);
+  dlg.exec();
+  idec = cb->currentItem();
+#else
+  KFileDialog dlg(currentDir() + filename, filter, 0, 0, true);
+  dlg.exec();
+#endif
+  return dlg.selectedURL();
+}
+
+void KplDoc::getFile(QLineEdit* e)
+{
+  QString s = KFileDialog::getOpenFileName(currentDir(), "*.so\n*");
+  if (!s.isEmpty()) {
+    e->setText(s);
+    setCurrentDir(s);
+  }
+}
+
+bool KplDoc::loadPar(const KURL& url, QWidget* parent, double* p, int idec)
+{
+  bool success = false;
+  QString fn;
+#if KDE_VERSION >= 0x03015a
+  if (KIO::NetAccess::download(url, fn, parent)) {
+#else
+  if (KIO::NetAccess::download(url, fn)) {
+#endif
+    QFile f(fn);
+    if (f.open(IO_ReadOnly)) {
+      memset(p, 0, KPL_NPMAX * sizeof(double));
+      QTextStream t(&f);
+      int i = -1;
+      while (!t.eof() && (++i < KPL_NPMAX)){
+        QString line = t.readLine();
+        Utils::translate(line, idec);
+        p[i] = line.toDouble();
+      }
+      f.close();
+      setCurrentDir(url);
+      success = true;
+    } else
+      KMessageBox::error(parent,
+                         i18n("while trying to open parameter file"));
+    KIO::NetAccess::removeTempFile(fn);
+  }
+  return success;
+}
+
+void KplDoc::getPar(QWidget* parent, double* p)
+{
+  int idec = aut.iDec;
+  KURL url = getReadURL(idec, "", "*.par *.PAR\n*");
+  if (!url.isEmpty())
+    loadPar(url, parent, p, idec);
+}
+
+bool KplDoc::saveParameter(QWidget* parent, const KURL& url, double* p,
+                           double* err)
+{
+  QFile f(url.isLocalFile() ? url.path() : tmpFile());
+  bool success = f.open(IO_WriteOnly);
+  if (success) {
+    QTextStream t(&f);
+    for (int i = 0; i < KPL_NPMAX; i++) {
+      t << number(p[i]);
+      if (err)
+        t << separator() << number(err[i]);
+      t << "\n";
+    }
+    f.close();
+    setCurrentDir(url);
+    if (!url.isLocalFile())
+      copyTmp(f.name(), url);
+  } else
+    KMessageBox::error(parent, i18n("while trying to open parameter file"));
+  return success;
+}
+
+void KplDoc::saveFunPar(QWidget* parent, double* p)
+{
+  KURL url;
+  if (getWriteURL(parent, url, "*.par\n*"))
+    saveParameter(parent, url, p, 0);
+}
+
 void KplDoc::autoFit(ArrayItem* ad, FunItem* fd)
 {
   fd->pathy = aut.fitPath;
@@ -360,6 +523,7 @@ void KplDoc::autoFit(ArrayItem* ad, FunItem* fd)
     for (int i = 0; i < KPL_NPMAX; i++)
       s1.insert(i, number(aut.err.pErrMod[i]));
     config->writeEntry("ErrorModelParam", s1, ' ');
+    config->writeEntry("ErrorModelArgument", aut.err.errModArg);
   }
   FitDlg dlg((QWidget*) parent(), this, &arr, &fun, mode);
   dlg.fit();
@@ -371,9 +535,33 @@ void KplDoc::autoFit(ArrayItem* ad, FunItem* fd)
   return;
 }
 
-bool KplDoc::openDocument(const KURL& url, int type)
+QComboBox* KplDoc::decimalWidget(int idec, KFileDialog* dlg, QHBox* hbox,
+                                 unsigned mode)
 {
-  fType = (FileTypes) type;
+  new QLabel(i18n("Decimal symbol"), hbox);
+  QComboBox* cb = new QComboBox(false, hbox);
+  QStringList list;
+  list << "." << ",";
+  cb->insertStringList(list);
+  cb->setCurrentItem(idec);
+  cb->setFixedWidth(80);
+#if QT_VERSION >= 0x030100
+  (new QLabel("", hbox))->setSizePolicy(QSizePolicy::MinimumExpanding,
+                                        QSizePolicy::Minimum);
+#else
+  (new QLabel("", hbox))->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
+                                        QSizePolicy::Minimum));
+#endif
+  hbox->setSpacing(dlg->spacingHint());
+  dlg->setOperationMode(KFileDialog::Opening);
+  dlg->setCaption(i18n("Open"));
+  dlg->setMode(mode);
+  return cb;
+}
+
+bool KplDoc::openDocument(const KURL& url, int idec, FileType type)
+{
+  fType = type;
   if (fType == Unknown) {
     QString s = url.fileName();
     int pos = s.findRev(".plo", -1, false);
@@ -399,7 +587,8 @@ bool KplDoc::openDocument(const KURL& url, int type)
     newItem(KplItem::Array);
     ArrayItem* ad = (ArrayItem*) itd.last();
     ad->url = url;
-    ad->n = ad->nrows = ArrayItem::readFile(url, &ad->ncols, &ad->x);
+    ad->idec = idec;
+    ad->n = ad->nrows = ArrayItem::readFile(url, &ad->ncols, &ad->x, ad->idec);
     success = ad->n;
     if (success) {
       if (itd.count() == 2) {
@@ -434,22 +623,27 @@ bool KplDoc::openDocument(const KURL& url, int type)
       }
     }
   } else {
-    QString tmpFile;
-    success = KIO::NetAccess::download(url, tmpFile);
+    QString tmpfile;
+#if KDE_VERSION >= 0x03015a
+    success = KIO::NetAccess::download(url, tmpfile, (QWidget*) parent());
+#else
+    success = KIO::NetAccess::download(url, tmpfile);
+#endif
     if (success) {
-      QFileInfo f2(tmpFile);
+      QFileInfo f2(tmpfile);
       success = f2.isReadable();
       if (success) {
         if (!aut.addData)
           newDocument();
-        KSimpleConfig plo(tmpFile, true);
+        KSimpleConfig plo(tmpfile, true);
         plo.setGroup("Items");
         if (int nItems = QMAX(plo.readNumEntry("nitems"), 0)) {
           const char *itemName[] = {"FRAMEITEM", "ARRAYITEM", "FUNITEM",
                                     "PARFUNITEM", "SPLINEITEM", "ARRAY3DITEM",
                                     "FUN3DITEM", "LEGENDITEM", "TEXTITEM",
                                     "LINEITEM", "ARROWITEM", "ARCITEM",
-                                    "RECTITEM", "ELLIPSEITEM"};
+                                    "RECTITEM", "ELLIPSEITEM", "IMAGEITEM",
+                                    "SCALEBARITEM"};
           const KplItem::ItemTypes iTypes[] = {KplItem::Frame, KplItem::Array,
                                                KplItem::Function,
                                                KplItem::ParFunction,
@@ -459,7 +653,8 @@ bool KplDoc::openDocument(const KURL& url, int type)
                                                KplItem::Legend, KplItem::Text,
                                                KplItem::Line, KplItem::Arrow,
                                                KplItem::Arc, KplItem::Rectangle,
-                                               KplItem::Ellipse};
+                                               KplItem::Ellipse, KplItem::Image,
+                                               KplItem::ScaleBar};
           setURL(url);
           for (int i = 0; i < nItems; i++) {
             QString s;
@@ -512,12 +707,18 @@ bool KplDoc::openDocument(const KURL& url, int type)
                 break;
               case KplItem::Ellipse:
                 itd.append(new EllipseItem(&plo, &aut));
+                break;
+              case KplItem::Image:
+                itd.append(new ImageItem(&plo, url));
+                break;
+              case KplItem::ScaleBar:
+                itd.append(new ScaleBarItem(&plo, &aut));
             }
           }
         } else
           success = false;
       }
-      KIO::NetAccess::removeTempFile(tmpFile);
+      KIO::NetAccess::removeTempFile(tmpfile);
     }
   }
   if (success)
@@ -569,17 +770,19 @@ bool KplDoc::redoAllowed() const
   return (itb.current() != itb.getLast());
 }
 
-void KplDoc::slotIOfinished()
+void KplDoc::slotIOfinished(KIO::Job* job)
 {
-  if (job->error()) {
-    job->showErrorDialog();
-    QFileInfo fi(job->destURL().path());
-    if (fi.extension().lower() == "plo") {
-      setModified();
-      emit modelChanged(false, false);
+  if (job) {
+    if (job->error()) {
+      job->showErrorDialog();
+      QFileInfo fi(((KIO::FileCopyJob*) job)->destURL().path());
+      if (fi.extension().lower() == "plo") {
+        setModified();
+        emit modelChanged(false, false);
+      }
     }
+    remove(((KIO::FileCopyJob*) job)->srcURL().path());
   }
-  remove(tmpFile());
 }
 
 void KplDoc::backupItems(bool list)
