@@ -6,7 +6,7 @@
     data sets and functions.
 
     begin                : Wed Sep 08 1999
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -22,7 +22,7 @@
 #include <qcolor.h>
 #include "kplgraph.h"
 
-KplGraph::KplGraph() : logx(false), logy(false), kSymb(0), relSize(1.0),
+KplGraph::KplGraph() : logx(false), logy(false), kSymb(0), relSiz(1.0),
  fxn(1.0), fyn(1.0), devType(Screen)
 {
 }
@@ -36,6 +36,11 @@ KplGraph::DevTypes KplGraph::type() const
   return devType;
 }
 
+double KplGraph::relSize() const
+{
+  return relSiz;
+}
+
 int KplGraph::prenum(double a, int ndig, QString& label)
 {
   label.sprintf("%.*f", QMAX(0, ndig), a);
