@@ -2,7 +2,7 @@
                           residualsdlg.cpp  -  description
                              -------------------
     begin                : Sun Jul 15 2001
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -21,14 +21,13 @@
 #include <kconfig.h>
 #include <kapp.h>
 #include <kglobal.h>
-#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 1)
+#if KDE_VERSION >= 220
 #include <kprinter.h>
 #else
 #include <qprinter.h>
 #endif
 #include "residualsdlg.h"
 #include "kplwidget.h"
-#include "utils.h"
 #include "kpldoc.h"
 
 ResidualsDlg::ResidualsDlg(QWidget* _parent, KplDoc* model,
@@ -48,7 +47,7 @@ ResidualsDlg::ResidualsDlg(QWidget* _parent, KplDoc* model,
   vbox->addWidget(new KplWidget(frame, items, w, h, zoom));
   QWidget* d = QApplication::desktop();
   setMaximumSize(QMIN(w + 50, d->width() - 20),
-                 QMIN(h + 60, d->height() - 40));
+                 QMIN(h + 63, d->height() - 40));
 }
 
 ResidualsDlg::~ResidualsDlg()
@@ -57,7 +56,7 @@ ResidualsDlg::~ResidualsDlg()
 
 void ResidualsDlg::slotUser1()
 {
-#if (KDE_VERSION_MAJOR > 2) || (KDE_VERSION_MINOR > 1)
+#if KDE_VERSION >= 220
   KPrinter printer;
 #else
   QPrinter printer;
@@ -69,6 +68,6 @@ void ResidualsDlg::slotUser1()
 void ResidualsDlg::slotUser2()
 {
   KURL url;
-  if (Utils::getWriteURL(this, url, "*.plo\n*", m))
+  if (m->getWriteURL(this, url, "*.plo\n*"))
     m->saveDocument(url, items);
 }
