@@ -2,7 +2,7 @@
                           splfitdlg.cpp  -  description
                              -------------------
     begin                : Fri Mar 1 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -190,7 +190,8 @@ int SplFitDlg::fit(int k, double f, double xmin, double xmax, bool errcol,
   QArray<double> c(nest);
   int nk;
   for (int i = 0; i < ad->n; i++) {
-    sigv[i] = errcol ? ev[i] : (fktErrMod(yv[i], err.pErrMod));
+    sigv[i] = errcol ? ev[i] : (fktErrMod(err.errModArg ? yv[i] : xv[i],
+                                          err.pErrMod));
     if (sigv[i] <= 0.0) {
       ier = 4;
       break;
@@ -281,7 +282,7 @@ void SplFitDlg::slotUser2()
   fd->sx = "x";
   fd->sy = i18n("Residuals");
   items.append(fd);
-  ArrayItem* ar = new ArrayItem(true, ad->symb, "0", 1.0, 1.0, 0, 1, 2,
+  ArrayItem* ar = new ArrayItem(true, false, ad->symb, "0", 1.0, 1.0, 0, 1, 2,
                                 0, xv.size(), 1, "", true);
   ar->color = ad->color;
   for (unsigned i = 0; i < xv.size(); i++) {
