@@ -2,7 +2,7 @@
                           splinedlg.cpp  -  description
                              -------------------
     begin                : Sun Mar 3 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -24,6 +24,7 @@
 #include <kcolorbtn.h>
 #include <klocale.h>
 #include <kmessagebox.h>
+#include "brushdlg.h"
 #include "kpldoc.h"
 #include "kpldoubleedit.h"
 #include "kplspinbox.h"
@@ -31,7 +32,6 @@
 #include "splinedlg.h"
 #include "splineitem.h"
 #include "symboldlg.h"
-#include "utils.h"
 
 SplineDlg::SplineDlg(QWidget* _parent, KplDoc* model, SplineItem* fd0) :
  KDialogBase(Plain, i18n("Smoothing spline"),
@@ -70,17 +70,16 @@ SplineDlg::SplineDlg(QWidget* _parent, KplDoc* model, SplineItem* fd0) :
   grid->addWidget(new QLabel("xmax", g), 0, 3);
   grid->addWidget(exMax = new KplDoubleEdit(fd->xmax, g, frm, prec), 0, 4);
   exMax->setMinimumWidth(w);
-  hbox = new QHBoxLayout(1);
-  grid->addLayout(hbox, 1, 0);
-  QLabel* l = new QLabel("D", g);
 #if KDE_VERSION_MAJOR > 2
-  l->setFont(QFont("Symbol", 12, QFont::Normal, false));
+  grid->addWidget(new QLabel(QString::fromUtf8("\316\224x"), g), 1, 0);
 #else
+  grid->addLayout(hbox = new QHBoxLayout(1), 1, 0);
+  QLabel* l = new QLabel("D", g);
   l->setFont(QFont("Symbol", 12, QFont::Normal, false, QFont::AnyCharSet));
-#endif
   hbox->addWidget(l);
   hbox->addWidget(new QLabel("x", g));
   hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
+#endif
   grid->addWidget(exStep = new KplDoubleEdit(fd->dx, g, frm, prec), 1, 1);
   exStep->setMinimumWidth(w);
   grid->addWidget(new QLabel(i18n("x normalization"), g), 2, 0);
@@ -93,16 +92,16 @@ SplineDlg::SplineDlg(QWidget* _parent, KplDoc* model, SplineItem* fd0) :
   grid->addWidget(new QLabel(i18n("Symbol"), g), 0, 0);
   grid->addWidget(symb = new KplSpinBox(-17, 9, 1, g), 0, 1);
   symb->setValue(fd->symb);
-  SymbolButton *bSymb = new SymbolButton(g, symb);
-  bSymb->setFixedWidth(50);
-  grid->addWidget(bSymb, 0, 2);
+  grid->addWidget(new SymbolButton(g, symb), 0, 2);
+  grid->addWidget(new QLabel(i18n("Color"), g), 0, 4);
+  grid->addWidget(colData = new KColorButton(fd->color, g), 0, 5);
+  colData->setMinimumWidth(50);
   grid->addWidget(new QLabel(i18n("Size"), g), 1, 0);
   grid->addWidget(eRelSize = new KplSpinBox(10, 800, 1, g), 1, 1);
   eRelSize->setValue(qRound(100 * fd->relsiz));
   eRelSize->setSuffix(" %");
-  grid->addWidget(new QLabel(i18n("Color"), g), 1, 4);
-  grid->addWidget(colData = new KColorButton(fd->color, g), 1, 5);
-  colData->setMinimumWidth(50);
+  grid->addWidget(new QLabel(i18n("Fill pattern"), g), 1, 4);
+  grid->addWidget(fill = new BrushButton(g, fd->fillStyle), 1, 5);
   setHelp("SEC-SPLINE");
   enableButton(User2, fd->nk && (fd->kdeg == 3));
   resize(minimumSize());
@@ -124,6 +123,7 @@ void SplineDlg::getTemp(SplineItem* fdt)
   fdt->color = colData->color().rgb();
   fdt->symb = symb->interpretedValue();
   fdt->relsiz = 0.01 * eRelSize->interpretedValue();
+  fdt->fillStyle = fill->value();
 }
 
 void SplineDlg::getValues(bool ok)
@@ -145,7 +145,7 @@ void SplineDlg::slotUser1()
   SplineItem fdt(*fd);
   getTemp(&fdt);
   KURL url;
-  if (Utils::getWriteURL(this, url, "*.dat\n*", m)) {
+  if (m->getWriteURL(this, url, "*.dat\n*")) {
     QFile f(url.isLocalFile() ? url.path() : m->tmpFile());
     if (f.open(IO_WriteOnly)) {
       QTextStream ts(&f);
@@ -153,7 +153,7 @@ void SplineDlg::slotUser1()
       f.close();
       m->setCurrentDir(url);
       if (!url.isLocalFile())
-        m->copyTmp(url);
+        m->copyTmp(f.name(), url);
     } else
       KMessageBox::error(this, i18n("while trying to open"));
   }
