@@ -2,7 +2,7 @@
                           splinedlg.h  -  description
                              -------------------
     begin                : Sun Mar 3 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -22,10 +22,11 @@
 
 class QComboBox;
 class KColorButton;
+class BrushButton;
 class KplDoc;
-class SplineItem;
 class KplDoubleEdit;
 class KplSpinBox;
+class SplineItem;
 
 /**
   * Spline settings dialog class. Provides a dialog for the settings
@@ -60,6 +61,7 @@ protected:
   KplDoc* m;
   SplineItem *fd;
   QComboBox *eDeriv;
+  BrushButton *fill;
   KplDoubleEdit *exLow, *exMin, *exMax, *exStep, *efx, *efy;
   KplSpinBox *symb, *eRelSize;
   KColorButton* colData;
