@@ -2,7 +2,7 @@
                           splineitem.cpp  -  description
                              -------------------
     begin                : Wed Feb 27 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -72,12 +72,12 @@ SplineItem::SplineItem(KSimpleConfig* plo, Kpl::AutoStruct* aut,
     c[i] = plo->readDoubleNumEntry(s.sprintf("c%i", i));
 }
 
-SplineItem::SplineItem(bool act, int sym, const QString& col, double xn,
-                       double yn, double xmi, double xma, double d_x,
-                       int deriv, double low, double relSize) :
- ScaledItem(act, sym, col, xn, yn, relSize), logxo(false), kdeg(3), nk(0),
- nderiv(deriv), kdego(3), nko(0), nderivo(0), xmin(xmi), xmax(xma), dx(d_x),
- xlow(low), xmino(0.0), xmaxo(0.0), dxo(0.0), xlowo(0.0)
+SplineItem::SplineItem(bool act, int fill, int sym, const QString& col,
+                       double xn, double yn, double xmi, double xma,
+                       double d_x, int deriv, double low, double relSize) :
+ ScaledItem(act, fill, sym, col, xn, yn, relSize), logxo(false), kdeg(3),
+ nk(0), nderiv(deriv), kdego(3), nko(0), nderivo(0), xmin(xmi), xmax(xma),
+ dx(d_x), xlow(low), xmino(0.0), xmaxo(0.0), dxo(0.0), xlowo(0.0)
 {
 }
 
@@ -125,10 +125,10 @@ void SplineItem::draw(KplGraph* g)
 {
   if (nk && active) {
     setProperties(g);
-    double sav = g->setRelSize(relsiz);
+    double sav = g->relSize();
     g->setRelSize(relsiz * sav);
     int n = calcTable(g->logx);
-    g->plArray(xv, yv, fx, fy, n);
+    g->plArray(xv, yv, fx, fy, n, fillStyle);
     g->setRelSize(sav);
   }
 }
