@@ -2,7 +2,7 @@
                           splineitem.h  -  description
                              -------------------
     begin                : Wed Feb 27 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -46,6 +46,7 @@ public:
   /**
     * Constructor. Initializes corresponding to arguments.
     * @param active true for visible item.
+    * @param fillStyle > 0 for filled polygon representation.
     * @param symb 0 for circles, < 0 for different markers,
     * >0 for different line types.
     * @param color color for function plot.
@@ -58,9 +59,9 @@ public:
     * @param low lower limit for spline integral.
     * @param relSize relative size of lines or symbols.
     */
-  SplineItem(bool active, int symb, const QString& color, double fx, double fy,
-             double xmin, double xmax, double dx, int deriv, double low,
-             double relSize = 1.0);
+  SplineItem(bool active, int fillStyle, int symb, const QString& color,
+             double fx, double fy, double xmin, double xmax, double dx,
+             int deriv, double low, double relSize = 1.0);
   /** Destructor */
   ~SplineItem();
   /** Assignment operator */
@@ -72,7 +73,7 @@ public:
   virtual void setText(KplCheckListItem* it, bool* arrays, bool* funcs) const;
   virtual int editItem(QWidget* parent, KplDoc* m, int i);
   virtual KplItem* copy() const;
-  virtual void expoItem(int* iext, int* ieyt, double* fxt, double* fyt) const;
+  virtual void expoItem(int* iex, int* iey, double* fx, double* fy) const;
   virtual void minMax(double* xmi, double* xma, double* ymi, double* yma) const;
   virtual void exportTable(QTextStream& ts, KplDoc* m) const;
 
