@@ -2,7 +2,7 @@
                           threeddlg.cpp  -  description
                              -------------------
     begin                : Thu Apr 25 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -16,34 +16,35 @@
  ***************************************************************************/
 
 #include <math.h>
-#include <qlayout.h>
-#include <qgroupbox.h>
-#include <qlabel.h>
-#include <qlineedit.h>
 #include <qbuttongroup.h>
-#include <qradiobutton.h>
 #include <qcheckbox.h>
 #include <qfile.h>
+#include <qgroupbox.h>
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qlineedit.h>
+#include <qradiobutton.h>
 #include <kapp.h>
 #if KDE_VERSION_MAJOR < 3
 #include <qmultilineedit.h>
 #endif
-#include <klocale.h>
 #include <kcolorbtn.h>
-#include <kmessagebox.h>
-#include <kiconloader.h>
 #include <kfiledialog.h>
+#include <kiconloader.h>
+#include <klocale.h>
+#include <kmessagebox.h>
+#include "arrayitem.h"
+#include "editdlg.h"
+#include "editparameterdlg.h"
+#include "edittextdlg.h"
+#include "funcdlg.h"
+#include "kpldoc.h"
 #include "kpldoubleedit.h"
+#include "kplgraph.h"
 #include "kplspinbox.h"
 #include "threeddlg.h"
 #include "threeditem.h"
-#include "kpldoc.h"
-#include "kplgraph.h"
 #include "utils.h"
-#include "funcdlg.h"
-#include "edittextdlg.h"
-#include "arrayitem.h"
-#include "editdlg.h"
 
 ThreeDDlg::ThreeDDlg(QWidget* _parent, KplDoc* model, ThreeDItem *td0,
                      const QString& caption, const QString anchor,
@@ -241,16 +242,17 @@ void ThreeDDlg::initRange(QGridLayout* grid, int row)
   grid->addWidget(exMax = new KplDoubleEdit(td->xma, g1, frm, prec), row, 4);
   exMax->setMinimumWidth(w);
   grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row, 5);
-  QHBoxLayout* hbox = new QHBoxLayout(1);
-  grid->addLayout(hbox, row, 6);
-  QLabel* l = new QLabel("D", g1);
 #if KDE_VERSION_MAJOR > 2
-  l->setFont(QFont("Symbol", 12, QFont::Normal, false));
+  grid->addWidget(new QLabel(QString::fromUtf8("\316\224x"), g1), row, 6);
 #else
+  QHBoxLayout* hbox = new QHBoxLayout(1);
+  grid->addLayout(hbox = new QHBoxLayout(1), row, 6);
+  QLabel* l = new QLabel("D", g1);
   l->setFont(QFont("Symbol", 12, QFont::Normal, false, QFont::AnyCharSet));
-#endif
   hbox->addWidget(l);
   hbox->addWidget(new QLabel("x", g1));
+  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
+#endif
   grid->addWidget(exStep = new KplDoubleEdit(td->dx, g1, frm, prec), row, 7);
   exStep->setMinimumWidth(w);
   grid->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding), row, 8);
@@ -268,15 +270,16 @@ void ThreeDDlg::initRange(QGridLayout* grid, int row)
   grid->addWidget(new QLabel("zmax", g1), row, 3);
   grid->addWidget(ezMax = new KplDoubleEdit(td->zma, g1, frm, prec), row, 4);
   ezMax->setMinimumWidth(w);
+#if KDE_VERSION_MAJOR > 2
+  grid->addWidget(new QLabel(QString::fromUtf8("\316\224z"), g1), row, 6);
+#else
   grid->addLayout(hbox = new QHBoxLayout(1), row, 6);
   l = new QLabel("D", g1);
-#if KDE_VERSION_MAJOR > 2
-  l->setFont(QFont("Symbol", 12, QFont::Normal, false));
-#else
   l->setFont(QFont("Symbol", 12, QFont::Normal, false, QFont::AnyCharSet));
-#endif
   hbox->addWidget(l);
   hbox->addWidget(new QLabel("z", g1));
+  hbox->addItem(new QSpacerItem(20, 10, QSizePolicy::MinimumExpanding));
+#endif
   grid->addWidget(ezStep = new KplDoubleEdit(td->dz, g1, frm, prec), row, 7);
   ezStep->setMinimumWidth(w);
   grid->addWidget(logZ = new QCheckBox("log", g1), row, 9);
@@ -323,15 +326,8 @@ void ThreeDDlg::getValues()
   td->theta = eTheta->value();
   td->xref = eXref->value();
   td->yref = eYref->value();
-  QRadioButton* rb;
-  td->gridmode = -1;
-  do
-    rb = (QRadioButton*) bg1->find(++td->gridmode);
-  while (!rb->isChecked());
-  td->mode3D = -1;
-  do
-    rb = (QRadioButton*) bg2->find(++td->mode3D);
-  while (!rb->isChecked());
+  td->gridmode = bg1->id(bg1->selected());
+  td->mode3D = bg2->id(bg2->selected());
   td->showFrame = modeFrame->isChecked();
   td->relsiz = 0.01 * eRelSiz->interpretedValue();
   td->colFrame = colFrame->color().rgb();
@@ -533,7 +529,7 @@ void Fun3DDlg::getValues(bool ok)
 
 void Fun3DDlg::slotFile()
 {
-  Utils::getFile(fileName, m);
+  m->getFile(fileName);
 }
 
 void Fun3DDlg::slotFunc()
@@ -543,7 +539,7 @@ void Fun3DDlg::slotFunc()
 
 void Fun3DDlg::slotGetParameter()
 {
-  Utils::getPar(this, fdt->py, m);
+  m->getPar(this, fdt->py);
 }
 
 void Fun3DDlg::slotEditParameter()
@@ -556,14 +552,14 @@ void Fun3DDlg::slotEditParameter()
 
 void Fun3DDlg::slotSaveParameter()
 {
-  Utils::saveFunPar(this, fdt->py, m);
+  m->saveFunPar(this, fdt->py);
 }
 
 void Fun3DDlg::slotExportValues()
 {
   if (getTemp()) {
     KURL url;
-    if (Utils::getWriteURL(this, url, "*.dat\n*", m)) {
+    if (m->getWriteURL(this, url, "*.dat\n*")) {
       QFile f(url.isLocalFile() ? url.path() : m->tmpFile());
       if (f.open(IO_WriteOnly)) {
         QTextStream ts(&f);
@@ -571,7 +567,7 @@ void Fun3DDlg::slotExportValues()
         f.close();
         m->setCurrentDir(url);
         if (!url.isLocalFile())
-          m->copyTmp(url);
+          m->copyTmp(f.name(), url);
       } else
         KMessageBox::error(this, i18n("while trying to open"));
     }
@@ -606,7 +602,7 @@ void Fun3DDlg::slotUser1()
 {
   if (getTemp()) {
     KURL url;
-    if (Utils::getWriteURL(this, url, "*.dat\n*", m)) {
+    if (m->getWriteURL(this, url, "*.dat\n*")) {
       QFile f(url.isLocalFile() ? url.path() : m->tmpFile());
       if (f.open(IO_WriteOnly)) {
         QTextStream ts(&f);
@@ -614,7 +610,7 @@ void Fun3DDlg::slotUser1()
         f.close();
         m->setCurrentDir(url);
         if (!url.isLocalFile())
-          m->copyTmp(url);
+          m->copyTmp(f.name(), url);
       } else
         KMessageBox::error(this, i18n("while trying to open"));
     }
@@ -635,8 +631,13 @@ Array3DDlg::Array3DDlg(QWidget* _parent, KplDoc* model, Array3DItem *ad0) :
   if (!ad->internal)
     fileName->setText(ad->url.isLocalFile() ?  ad->url.path() : ad->url.url());
   fileName->setFrameStyle(QFrame::Panel | QFrame::Sunken);
+#if QT_VERSION >= 0x030100
+  fileName->setSizePolicy(QSizePolicy::MinimumExpanding,
+                          QSizePolicy::Minimum);
+#else
   fileName->setSizePolicy(QSizePolicy(QSizePolicy::MinimumExpanding,
                                       QSizePolicy::Minimum));
+#endif
   QPushButton* b = new QPushButton("...", g);
   b->setFixedWidth(28);
   hbox->addWidget(b);
@@ -709,13 +710,14 @@ void Array3DDlg::setSpinBoxes(Array3DItem* _ad)
   n->setValue(_ad->n);
 }
 
-bool Array3DDlg::load(KURL &url)
+bool Array3DDlg::load(KURL &url, int idec)
 {
   bool success = !url.isEmpty();
   if (success) {
     delete adt;
     adt = new Array3DItem;
     adt->url = url;
+    adt->idec = idec;
     success = readData();
   }
   return success;
@@ -754,7 +756,8 @@ bool Array3DDlg::readData(bool fromBuffer)
   if (fromBuffer)
     adt->nrows = ArrayItem::readFile(&buf, &adt->ncols, &adt->xv);
   else
-    adt->nrows = ArrayItem::readFile(adt->url, &adt->ncols, &adt->xv);
+    adt->nrows = ArrayItem::readFile(adt->url, &adt->ncols, &adt->xv,
+                                     adt->idec);
   if (adt->nrows) {
     int maxCol = adt->ncols - 1;
     adt->ix = QMIN(ad->ix, maxCol);
@@ -791,8 +794,9 @@ void Array3DDlg::slotNChanged(int na)
 
 void Array3DDlg::slotFile()
 {
-  KURL url = KFileDialog::getOpenURL(m->currentDir(), "*.dat *.DAT\n*");
-  if (load(url)) {
+  int idec = adt ? adt->idec : ad->idec;
+  KURL url = m->getReadURL(idec, (adt ? adt : ad)->url.fileName());
+  if (load(url, idec)) {
     fileName->setText(url.isLocalFile() ? url.path() : url.url());
     m->setCurrentDir(url);
     bReload->setDisabled(fileName->text().isEmpty());
@@ -806,7 +810,7 @@ void Array3DDlg::slotFile()
 void Array3DDlg::slotReload()
 {
   KURL url(fileName->text());
-  load(url);
+  load(url, adt ? adt->idec : ad->idec);
 }
 
 void Array3DDlg::slotInternal(bool state)
@@ -850,25 +854,25 @@ void Array3DDlg::slotInternal(bool state)
     }
     setSpinBoxes(adt);
     fileName->clear();
+    adt->idec = 0;
     delete a2;
   } else {
     Array3DItem* src = adt ? adt : ad;
     KURL url;
-    if (Utils::getWriteURL(this, url, "*.dat\n*", m)) {
+    if (m->getWriteURL(this, url, "*.dat\n*")) {
       QFile f(url.isLocalFile() ? url.path() : m->tmpFile());
       bool success = f.open(IO_WriteOnly);
       if (success) {
         QTextStream t(&f);
         for (int i = 0; i < src->nrows; i++)
-          for (int j = 0; j < src->ncols; j++) {
-            t << QString::number(src->xv[j][i]);
-            t << (j < (src->ncols - 1) ? " " : "\n");
-          }
+          for (int j = 0; j < src->ncols; j++)
+            t << m->number(src->xv[j][i])
+              << (j < (src->ncols - 1) ? " " : "\n");
         f.close();
         fileName->setText(url.isLocalFile() ? url.path() : url.url());
         m->setCurrentDir(url);
         if (!url.isLocalFile())
-          m->copyTmp(url);
+          m->copyTmp(f.name(), url);
       } else {
         KMessageBox::error(this, i18n("while trying to open file"));
         internal->setChecked(true);
@@ -944,8 +948,8 @@ void Array3DDlg::slotApply()
 void Array3DDlg::slotUser1()
 {
   KURL url(fileName->text());
-  if (load(url)) {
-    if (Utils::getWriteURL(this, url, "*.dat\n*", m)) {
+  if (load(url, adt ? adt->idec : ad->idec)) {
+    if (m->getWriteURL(this, url, "*.dat\n*")) {
       QFile f(url.isLocalFile() ? url.path() : m->tmpFile());
       if (f.open(IO_WriteOnly)) {
         QTextStream ts(&f);
@@ -971,7 +975,7 @@ void Array3DDlg::slotUser1()
         f.close();
         m->setCurrentDir(url);
         if (!url.isLocalFile())
-          m->copyTmp(url);
+          m->copyTmp(f.name(), url);
       } else
         KMessageBox::error(this, i18n("while trying to open"));
     }
