@@ -2,7 +2,7 @@
                           threeddlg.h  -  description
                              -------------------
     begin                : Thu Apr 25 2002
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -201,9 +201,11 @@ protected:
   void setSpinBoxes(Array3DItem* ad);
   /**
    * Loads file.
+   * @param url KURL of file.
+   * @param idec 0 for decimal point, 1 for comma.
    * @return true when when succeeded.
    */
-  bool load(KURL &url);
+  bool load(KURL &url, int idec);
   /**
    * Copies settings to 3D array object.
    * @param accept true for leaving the dialog, false for applying.
