@@ -2,7 +2,7 @@
                           threeditem.cpp  -  description
                              -------------------
     begin                : Tue Dec 25 2001
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -521,7 +521,7 @@ void ThreeDItem::axis3d(KplGraph* g, double rmi, double rma, double tic,
   double xr, yr, rl2;
   double rl = 0.0;
   double mant = 0.0;
-  double sav = g->setRelSize(1.0);
+  double sav = g->relSize();
   double d2 = 0.0125 * p3d.length;
   double xArr[2], yArr[2];
   g->setColData(colGrid);
@@ -694,7 +694,7 @@ void ThreeDItem::axis3d(KplGraph* g, double rmi, double rma, double tic,
 
 void ThreeDItem::rast3d(KplGraph* g)
 {
-  double sav = g->setRelSize(1.0);
+  double sav = g->relSize();
   g->setRelSize(1.414 * sav);
   g->setColData(colFrame);
   frame3d(g, (p3d.a11 * p3d.a22 < 0.0) ? 3 : 4);
@@ -743,7 +743,7 @@ void ThreeDItem::rast3d(KplGraph* g)
 
 void ThreeDItem::plar3d(KplGraph* g)
 {
-  double sav = g->setRelSize(1.0);
+  double sav = g->relSize();
   g->setRelSize(0.707 * sav);
   double x = 0.5 * scalx * fx * (logx ? log10(xma * xmi) : (xma + xmi));
   double z = 0.5 * scalz * fz * (logz ? log10(zma * zmi) : (zma + zmi));
@@ -1015,24 +1015,26 @@ int Fun3DItem::calcTable() const
 }
 
 Array3DItem::Array3DItem() : internal(false), ix(0), iy(0), iz(0), ie(0),
- istart(0), n(0), errbars(0), nrows(0), ncols(0), ixo(0), iyo(0), izo(0),
- ieo(0), istarto(0), no(0), errbarso(0), smf(0.0), smfo(0.0), xv(0)
+ istart(0), n(0), errbars(0), nrows(0), ncols(0), idec(0), ixo(0), iyo(0),
+ izo(0), ieo(0), istarto(0), no(0), errbarso(0), smf(0.0), smfo(0.0), xv(0)
 {
 }
 
 Array3DItem::Array3DItem(const Array3DItem& a) : ThreeDItem(a),
  internal(a.internal), ix(a.ix), iy(a.iy), iz(a.iz), ie(a.ie),
  istart(a.istart), n(a.n), errbars(a.errbars), nrows(a.nrows), ncols(a.ncols),
- ixo(a.ixo), iyo(a.iyo), izo(a.izo), ieo(a.ieo), istarto(a.istarto), no(a.no),
- errbarso(a.errbarso), smf(a.smf), smfo(a.smfo), xv(a.xv), url(a.url)
+ idec(a.idec), ixo(a.ixo), iyo(a.iyo), izo(a.izo), ieo(a.ieo),
+ istarto(a.istarto), no(a.no), errbarso(a.errbarso), smf(a.smf), smfo(a.smfo),
+ xv(a.xv), url(a.url)
 {
   xv = ArrayItem::copyX(a.xv, a.ncols, a.nrows);
 }
 
 Array3DItem::Array3DItem(Kpl::AutoStruct* aut) : ThreeDItem(aut),
  internal(false), ix(aut->ixAuto), iy(aut->iyAuto), iz(2), ie(aut->ieAuto),
- istart(0), n(0), errbars(aut->autoErr), nrows(0), ncols(0), ixo(0), iyo(0),
- izo(0), ieo(0), istarto(0), no(0), errbarso(0), smf(0.0), smfo(0.0), xv(0)
+ istart(0), n(0), errbars(aut->autoErr), nrows(0), ncols(0), idec(aut->iDec),
+ ixo(0), iyo(0), izo(0), ieo(0), istarto(0), no(0), errbarso(0), smf(0.0),
+ smfo(0.0), xv(0)
 {
 }
 
@@ -1043,10 +1045,12 @@ Array3DItem::Array3DItem(KSimpleConfig* plo, Kpl::AutoStruct* aut,
 {
   QString s = plo->readEntry("path", "");
   url = QUrl::isRelativeUrl(s) ? (uPlo.directory(false) + s) : s;
+  url.cleanPath();
   internal = plo->readNumEntry("internal", false);
   if (internal) {
     n = nrows = plo->readNumEntry("n");
     ncols = 3;
+    idec = 0;
     double* x1 = new double[ncols * nrows];
     xv = new double*[ncols];
     for (int j = 0; j < ncols; j++)
@@ -1059,7 +1063,8 @@ Array3DItem::Array3DItem(KSimpleConfig* plo, Kpl::AutoStruct* aut,
         xv[j][i] = (j < cnt) ? list[j].toDouble() : 0.0;
     }
   } else {
-    nrows = ArrayItem::readFile(url, &ncols, &xv);
+    idec = plo->readNumEntry("idec");
+    nrows = ArrayItem::readFile(url, &ncols, &xv, idec);
     if (!nrows) {
       KMessageBox::error(0, url.url());
         return;
@@ -1091,17 +1096,18 @@ Array3DItem::Array3DItem(bool act, bool lgx, bool lgy, bool lgz, bool frame,
                      const QString& sY, const QString& sZ, const QString& sH,
                      int i_x, int i_y, int i_z, int i_e, int i_start, int np,
                      int errb, double _smf, const KURL& u,
-                     Kpl::AutoStruct* aut, bool _int) :
+                     Kpl::AutoStruct* aut, bool _int, int iDec) :
  ThreeDItem(act, lgx, lgy, lgz,frame, ndx, ndy, ndz, mtx, mty, mtz, gm, gm3,
             ex, ey, ez, col_f, col_g, col_d, x_0, wx, y_0, hy, xmin, xmax,
             ymin, ymax, zmin, zmax, d_x, d_z, xt, yt, zt, relSize, phi_,
             theta_, b_x, b_y, b_z, xr, yr, f_x, f_y, f_z, sX, sY, sZ, sH, aut),
  internal(_int), ix(i_x), iy(i_y), iz(i_z), ie(i_e), istart(0), n(0),
- errbars(errb), smf(_smf), xv(0), url(u)
+ errbars(errb), idec(iDec), smf(_smf), xv(0), url(u)
 {
   if (internal) {
     n = nrows = np;
     ncols = 3;
+    idec = 0;
     int dim = ncols * nrows;
     double* x1 = new double[dim];
     memset(x1, 0, dim * sizeof(double));
@@ -1109,7 +1115,7 @@ Array3DItem::Array3DItem(bool act, bool lgx, bool lgy, bool lgz, bool frame,
     for (int j = 0; j < ncols; j++)
       xv[j] = &x1[j * nrows];
   } else {
-    nrows = ArrayItem::readFile(url, &ncols, &xv);
+    nrows = ArrayItem::readFile(url, &ncols, &xv, idec);
     if (!nrows) {
       KMessageBox::error(0, url.url());
         return;
@@ -1138,6 +1144,7 @@ Array3DItem& Array3DItem::operator=(const Array3DItem& a)
     errbars = a.errbars;
     nrows = a.nrows;
     ncols = a.ncols;
+    idec = a.idec;
     ixo = a.ixo;
     iyo = a.iyo;
     izo = a.izo;
@@ -1190,6 +1197,7 @@ void Array3DItem::writePlo(KSimpleConfig* plo, const KURL& uPlo, bool _abs,
       plo->writeEntry(s.sprintf("r%i", i), list, ' ');
     }
   } else {
+    plo->writeEntry("idec", idec);
     plo->writeEntry("istart", istart);
     plo->writeEntry("path", Utils::relPath(uPlo, url, _abs));
   }
@@ -1380,9 +1388,7 @@ void Array3DItem::calcTable() const
                       xmin, xmax, zmin, zmax, k, k, smf * smf * n, nest, nest,
                       nest, DBL_EPSILON, &nkx, tx.data(), &nkz, tz.data(),
                       c.data(), &fp, wrk.data(), lwrk, &ier);
-debug("surfit: ier = " + QString::number(ier) + ", fp = " + QString::number(fp));
       FitPack::bispev(tx, nkx, tz, nkz, c, k, k, xi, nx, zi, nz, yv[0], &ier);
-debug("bispev: ier = " + QString::number(ier));
     }
   }
   xmio = xmi;
