@@ -2,7 +2,7 @@
                           threeditem.h  -  description
                              -------------------
     begin                : Tue Dec 25 2001
-    copyright            : (C) 2003 by Werner Stille
+    copyright            : (C) 2004 by Werner Stille
     email                : stille@uni-freiburg.de
  ***************************************************************************/
 
@@ -448,6 +448,7 @@ public:
     * @param u URL of data file.
     * @param aut pointer to structure containing autoplot settings.
     * @param internal true for internal stored array data.
+    * @param idec 0 for decimal point, 1 for comma.
     */
   Array3DItem(bool act, bool lgx, bool lgy, bool lgz, bool frame, int ndx,
               int ndy, int ndz, int mtx, int mty, int mtz, int gm, int gm3,
@@ -461,7 +462,8 @@ public:
               double f_y, double f_z, const QString& sX, const QString& sY,
               const QString& sZ, const QString& sH, int i_x, int i_y, int i_z,
               int i_e, int i_start, int np, int errb, double _smf,
-              const KURL& u, Kpl::AutoStruct* aut, bool _int = false);
+              const KURL& u, Kpl::AutoStruct* aut, bool _int = false,
+              int idec = 0);
   /** Destructor */
   ~Array3DItem();
   /** Assignment operator */
@@ -481,7 +483,7 @@ protected:
    */
   void calcTable() const;
   bool internal;
-  int ix, iy, iz, ie, istart, n, errbars, nrows, ncols;
+  int ix, iy, iz, ie, istart, n, errbars, nrows, ncols, idec;
   mutable int ixo, iyo, izo, ieo, istarto, no, errbarso;
   double smf;
   mutable double smfo;
